cdef cfmin2struct(FunctionMinimum* cfmin):
    cfmin_struct = Struct(
            fval = cfmin.fval(),
            edm = cfmin.edm(),
            nfcn = cfmin.nfcn(),
            up = cfmin.up(),
            is_valid = cfmin.isValid(),
            has_valid_parameters = cfmin.hasValidParameters(),
            has_accurate_covar = cfmin.hasAccurateCovar(),
            has_posdef_covar = cfmin.hasPosDefCovar(),
            #forced to be posdef
            has_made_posdef_covar = cfmin.hasMadePosDefCovar(),
            hesse_failed = cfmin.hesseFailed(),
            has_covariance = cfmin.hasCovariance(),
            is_above_max_edm = cfmin.isAboveMaxEdm(),
            has_reached_call_limit = cfmin.hasReachedCallLimit()
        )
    return cfmin_struct


cdef minuitparam2struct(MinuitParameter mp):
    ret = Struct(
            number = mp.number(),
            name = mp.name(),
            value = mp.value(),
            error = mp.error(),
            is_const = mp.isConst(),
            is_fixed = mp.isFixed(),
            has_limits = mp.hasLimits(),
            has_lower_limit = mp.hasLowerLimit(),
            has_upper_limit = mp.hasUpperLimit(),
            lower_limit = mp.lowerLimit(),
            upper_limit = mp.upperLimit(),
        )
    return ret


cdef minoserror2struct(MinosError m):
    ret = Struct(
        lower = m.lower(),
        upper = m.upper(),
        is_valid = m.isValid(),
        lower_valid = m.lowerValid(),
        upper_valid = m.upperValid(),
        at_lower_limit = m.atLowerLimit(),
        at_upper_limit = m.atUpperLimit(),
        at_lower_max_fcn = m.atLowerMaxFcn(),
        at_upper_max_fcn = m.atUpperMaxFcn(),
        lower_new_min = m.lowerNewMin(),
        upper_new_min = m.upperNewMin(),
        nfcn = m.nfcn(),
        min = m.min()
        )
    return ret



from unittest import TestCase
from nose.tools import (raises, assert_equal, assert_true, assert_false,
    assert_almost_equal)
from iminuit import Minuit
from math import sqrt


def assert_array_almost_equal(actual, expected):
    """
    Helper function to test if all elements of a list of lists
    are almost equal.
    Like numpy.testing.assert_array_almost_equal,
    which we can't use here because we don't
    want to depend on numpy.
    """
    for row in range(len(actual)):
        for col in range(len(actual[0])):
            assert_almost_equal(actual[row][col], expected[row][col])


class Func_Code:
    def __init__(self, varname):
        self.co_varnames=varname
        self.co_argcount=len(varname)


class Func1:
    def __init__(self):
        pass

    def __call__(self, x, y):
        return (x-2.)**2 + (y-5.)**2 + 10


class Func2:
    def __init__(self):
        self.func_code = Func_Code(['x', 'y'])

    def __call__(self, *arg):
        return (arg[0]-2.)**2 + (arg[1]-5.)**2 + 10


def func3(x, y):
    return 0.2*(x-2.)**2 + (y-5.)**2 + 10


def func4(x, y, z):
    return 0.2*(x-2.)**2 + 0.1*(y-5.)**2 + 0.25*(z-7.)**2 + 10


def func5(x, long_variable_name_really_long_why_does_it_has_to_be_this_long, z):
    return (x**2)+(z**2)+long_variable_name_really_long_why_does_it_has_to_be_this_long**2


def functesthelper(f):
    m = Minuit(f, print_level=0, pedantic=False)
    m.migrad()
    val = m.values
    print val
    assert_almost_equal(val['x'], 2.)
    assert_almost_equal(val['y'], 5.)
    assert_almost_equal(m.fval, 10.)
    assert(m.matrix_accurate())
    assert(m.migrad_ok())


def test_f1():
    functesthelper(Func1())


def test_f2():
    functesthelper(Func2())


def test_f3():
    functesthelper(func3)


@raises(RuntimeError)
def test_typo():
    Minuit(func4, printlevel=0)
    #self.assertRaises(RuntimeError,Minuit,func4,printlevel=0)


def test_non_invertible():
    #making sure it doesn't crash
    def f(x, y):
        return (x*y)**2
    m = Minuit(f, pedantic=False, print_level=0)
    result = m.migrad()
    m.hesse()
    try:
        m.matrix()
        assert False #shouldn't reach here
    except RuntimeError as e:
        pass


def test_fix_param():
    m = Minuit(func4, print_level=0, pedantic=False)
    m.migrad()
    m.minos()
    val = m.values
    assert_almost_equal(val['x'], 2.)
    assert_almost_equal(val['y'], 5.)
    assert_almost_equal(val['z'], 7.)
    err = m.errors#second derivative
    m.print_all_minos()
    #now fix z = 10
    m = Minuit(func4, print_level=-1, y=10., fix_y=True, pedantic=False)
    m.migrad()
    val = m.values
    assert_almost_equal(val['x'], 2.)
    assert_almost_equal(val['y'], 10.)
    assert_almost_equal(val['z'], 7.)
    assert_almost_equal(m.fval, 10.+2.5)
    free_param = m.list_of_vary_param()
    fix_param = m.list_of_fixed_param()
    assert_true('x' in free_param)
    assert_false('x' in fix_param)
    assert_true('y' in fix_param)
    assert_false('y' in free_param)
    assert_false('z' in fix_param)


def test_fitarg():
    m = Minuit(func4, print_level=-1, y=10., fix_y=True, limit_x=(0, 20.),
        pedantic=False)
    fitarg = m.fitarg
    assert_almost_equal(fitarg['y'], 10.)
    assert_true(fitarg['fix_y'])
    assert_equal(fitarg['limit_x'], (0, 20))
    m.migrad()

    fitarg = m.fitarg

    assert_almost_equal(fitarg['y'], 10.)
    assert_almost_equal(fitarg['x'], 2., places=2)
    assert_almost_equal(fitarg['z'], 7., places=2)

    assert_true('error_y' in fitarg)
    assert_true('error_x' in fitarg)
    assert_true('error_z' in fitarg)

    assert_true(fitarg['fix_y'])
    assert_equal(fitarg['limit_x'], (0, 20))


def test_minos_all():
    m = Minuit(func3, pedantic=False, print_level=0)
    m.migrad()
    m.minos()
    assert_almost_equal(m.merrors[('x', -1.0)], -sqrt(5))
    assert_almost_equal(m.merrors[('x', 1.0)], sqrt(5))
    assert_almost_equal(m.merrors[('y', 1.0)], 1.)


def test_minos_single():
    m = Minuit(func3, pedantic=False, print_level=0)
    m.migrad()
    m.minos('x')
    assert_almost_equal(m.merrors[('x', -1.0)], -sqrt(5))
    assert_almost_equal(m.merrors[('x', 1.0)], sqrt(5))


def test_minos_single_fixed():
    m = Minuit(func3, pedantic=False, print_level=0, fix_x=True)
    m.migrad()
    ret = m.minos('x')
    assert_equal(ret, None)
    #assert_almost_equal(m.merrors[('x',-1.0)],-sqrt(5))
    #assert_almost_equal(m.merrors[('x',1.0)],sqrt(5))


@raises(RuntimeError)
def test_minos_single_no_migrad():
    m = Minuit(func3, pedantic=False, print_level=0)
    m.minos('x')


@raises(RuntimeError)
def test_minos_single_nonsense_variable():
    m = Minuit(func3, pedantic=False, print_level=0)
    m.migrad()
    m.minos('nonsense')


def test_fixing_long_variablename():
    m = Minuit(func5, pedantic=False, print_level=0,
    fix_long_variable_name_really_long_why_does_it_has_to_be_this_long=True,
    long_variable_name_really_long_why_does_it_has_to_be_this_long=0)
    m.migrad()


def test_initalvalue():
    m = Minuit(func3, pedantic=False, x=1., y=2., error_x=3., print_level=0)
    assert_almost_equal(m.args[0], 1.)
    assert_almost_equal(m.args[1], 2.)
    assert_almost_equal(m.values['x'], 1.)
    assert_almost_equal(m.values['y'], 2.)
    assert_almost_equal(m.errors['x'], 3.)


def test_mncontour():
    m = Minuit(func3, pedantic=False, x=1., y=2., error_x=3., print_level=0)
    m.migrad()
    xminos, yminos, ctr = m.mncontour('x', 'y', numpoints=30)
    xminos_t = m.minos('x')['x']
    yminos_t = m.minos('y')['y']
    assert_almost_equal(xminos.upper, xminos_t.upper)
    assert_almost_equal(xminos.lower, xminos_t.lower)
    assert_almost_equal(yminos.upper, yminos_t.upper)
    assert_almost_equal(yminos.lower, yminos_t.lower)
    assert_equal(len(ctr), 30)
    assert_equal(len(ctr[0]), 2)


def test_mncontour_sigma():
    m = Minuit(func3, pedantic=False, x=1., y=2., error_x=3., print_level=0)
    m.migrad()
    xminos, yminos, ctr = m.mncontour('x', 'y', numpoints=30, sigma=2.0)
    xminos_t = m.minos('x', sigma=2.0)['x']
    yminos_t = m.minos('y', sigma=2.0)['y']
    assert_almost_equal(xminos.upper, xminos_t.upper)
    assert_almost_equal(xminos.lower, xminos_t.lower)
    assert_almost_equal(yminos.upper, yminos_t.upper)
    assert_almost_equal(yminos.lower, yminos_t.lower)
    assert_equal(len(ctr), 30)
    assert_equal(len(ctr[0]), 2)


def test_contour():
    #FIXME: check the result
    m = Minuit(func3, pedantic=False, x=1., y=2., error_x=3., print_level=0)
    m.migrad()
    m.contour('x', 'y')


def test_profile():
    #FIXME: check the result
    m = Minuit(func3, pedantic=False, x=1., y=2., error_x=3., print_level=0)
    m.migrad()
    m.profile('y')


def test_mnprofile():
    #FIXME: check the result
    m = Minuit(func3, pedantic=False, x=1., y=2., error_x=3., print_level=0)
    m.migrad()
    m.mnprofile('y')

@raises(RuntimeError)
def test_printfmin_uninitialized():
    #issue 85
    def f(x): return 2 + 3 * x
    fitter = Minuit(f)
    fitter.print_fmin()

@raises(ValueError)
def test_reverse_limit():
    #issue 94
    def f(x,y,z):
        return (x-2)**2 + (y-3)**2 + (z-4)**2
    m = Minuit(f, limit_x=(3., 2.))
    m.migrad()

class TestErrorMatrix(TestCase):

    def setUp(self):
        self.m = Minuit(func3, print_level=0, pedantic=False)
        self.m.migrad()

    def test_error_matrix(self):
        actual = self.m.matrix()
        expected = [[5., 0.], [0., 1.]]
        assert_array_almost_equal(actual, expected)

    def test_error_matrix_correlation(self):
        actual = self.m.matrix(correlation=True)
        expected = [[1., 0.], [0., 1.]]
        assert_array_almost_equal(actual, expected)



# -*- coding: utf-8 -*-
# <nbformat>3.0</nbformat>

# <codecell>

%pylab inline
from iminuit import Minuit, describe, Struct

# <markdowncell>

# ##Really Quick Start
# Let go through a quick course about how to minimize things. If you use PyMinuit before you will find that iminuit is very similar to PyMinuit.
# 
# Our first example is about trying to minimize a simple function:
# 
# $f(x,y,z) = (x-1)^2 + (y-2)^2 + (z-3)^2 - 1$
# 
# We know easily that the answer has is
# $x=1$, $y=2$, $z=3$ and the minimum value should be $-1$

# <codecell>

def f(x,y,z):
    return (x-1.)**2 + (y-2*x)**2 + (z-3.*x)**2 -1.

# <markdowncell>

# iminuit relies on Python introspection. If you wonder what kind of function iminuit sees, you can use

# <codecell>

describe(f) 

# <markdowncell>

# ###Contruct Minuit Object
# To minimize we need to construct a Minuit object

# <codecell>

m=Minuit(f, x=2, error_x=0.2, limit_x=(-10.,10.), y=3., fix_y=True, print_level=1)

# <markdowncell>

# The initial value/error are optional but it's nice to do it
# and here is how to use it
# 
# - `x=2` set intial value of `x` to 2
# - `error_x=0.2` set the initial stepsize
# - `limit_x = (-1,1)` set the range for `x`
# - `y=2`, `fix_y=True` fix `y` value to 2
# 
# We did not put any constain on z. Minuit will howerver warn you about missig initial error/step(using python builtin warning).

# <markdowncell>

# ####Check that initial setting you put in is what you want

# <codecell>

m.print_param()#or call print_initial_param
#bonus: if you click the + button on the top left corner it will show latex table
#which you can copy and paste to your beamer/latex document

# <markdowncell>

# 
# 
# ###Run Migrad
# 
# Migrad performs Variable Metric Minimization. In a nutshell, it combines steepest descends algorithm along with line search strategy. Migrad is very popular in high energy physics field because of its robustness.

# <codecell>

#Minimize
m.migrad();
#notice also in your prompt it prints out progress

# <markdowncell>

# migrad summary table give you a nice overview of fit status. 
# 
# - All blocks should be green.
# - Red means something bad. 
# - Yellow is a caution(fit is good but you ran over call limit)
# 
# You can use the return value of `migrad()` to check fit status. Most important field is `is_valid`.

# <markdowncell>

# ###Accessing values and errors
# ####Accessing Values

# <codecell>

#and this is how you get the the value
print 'parameters', m.parameters
print 'args', m.args
print 'value', m.values

# <markdowncell>

# ####Error(parabolic)

# <codecell>

#and the error
print 'error', m.errors

# <markdowncell>

# ####Function minimum

# <codecell>

#and function value at the minimum
print 'fval', m.fval
#Tip: you can also obtain value at current state by
print 'current state', f(*m.args)

# <markdowncell>

# ####Correlation and Covariance Matrix

# <codecell>

#covariance, correlation matrix
#remember y is fixed
print 'covariance', m.covariance
print 'matrix()', m.matrix() #covariance
print 'matrix(correlation=True)', m.matrix(correlation=True) #correlation
m.print_matrix() #correlation
#again click the + button on the top left corner for latex code

# <markdowncell>

# ####Fit status

# <codecell>

#get mimization status
print m.get_fmin()
print m.get_fmin().is_valid

# <markdowncell>

# ###Contour and $\chi^2$/Likelihood profile
# $\chi^2$ and contour can be obtained easily

# <codecell>

#minos contour
xminos, zminos, ctr = m.mncontour('x','z')
fill(*zip(*ctr)) #looks kinda ugly, right?

# <codecell>

#drawing it nicely
m.draw_mncontour('x','z', nsigma=4);

# <codecell>

#or you can get the gridded data
x, y, g, r = m.mncontour_grid('x','z', nsigma=3) # r is the raw data
pcolormesh(x,y,g)
colorbar()

# <codecell>

#1D value Scan
x,y = m.profile('x',subtract_min=True);
plot(x,y) #if you have matplotlib

# <codecell>

#we also provide convenience wrapper for drawing it
m.draw_profile('x');

# <codecell>

#2d contour NOT minos contour
x,y,z = m.contour('x','y',subtract_min=True)
cs = contour(x,y,z)
clabel(cs)

# <codecell>

#or a convenience wrapper
m.draw_contour('x','z');

# <markdowncell>

# ###Hesse and Minos

# <markdowncell>

# ####Hesse
# Hesse find the error by finding the inverse of second derivative matrix(hessian). The error assume parabolic shape at the minimum. Hesse error is symmetric by construct. Hesse is always called at the end of migrad to get the error. You normally don't need to call it manually.

# <codecell>

m.hesse()

# <markdowncell>

# ####minos
# 
# minos multidimensionally scan likelihood/$\chi^2$ until to find the contour where the value of the cost function increase by `UP`(see `set_up`). It takes really long time but give the correct error(unless it fails). 

# <codecell>

m.minos() #call m.minos('x') if you need minos error for just 1 variable
print m.get_merrors()['x']
print m.get_merrors()['x'].lower
print m.get_merrors()['x'].upper

# <markdowncell>

# ###Printing Out Nice Tables
# you can force use print_* to do various html display

# <codecell>

m.print_param()
m.print_matrix()

# <markdowncell>

# ##Alternative Ways to define function
# ###Cython
# If you want speed with minimal code change this is the way to do it. This is a quick way to use cython. For a hard core cython see hard-core-tutorial.ipynb.

# <codecell>

#sometimes we want speeeeeeed
%load_ext cythonmagic
%pylab inline
from iminuit import Minuit, describe, Struct

# <codecell>

%%cython --force
cimport cython
import numpy as np
cimport numpy as np #overwritten those from python with cython

#@cython.binding(True) this works too
@cython.embedsignature(True)#dump signature in pydoc so describe can extract signature
def cython_f(double x,double y,double z):
    return (x-1.)**2 + (y-2.)**2 + (z-3.)**2 -1.

# <codecell>

#you can always see what iminuit will see
print describe(cython_f)

# <codecell>

m = Minuit(cython_f)
m.migrad()
print m.values

# <markdowncell>

# ###Callable object ie: __call__
# This is useful if you need to bound your object to some data

# <codecell>

%pylab inline
from iminuit import Minuit, describe, Struct

# <codecell>

x = [1,2,3,4,5]
y = [2,4,6,8,10]# y=2x
class StraightLineChi2:
    def __init__(self,x,y):
        self.x = x
        self.y = y
    def __call__(self,m,c): #lets try to find slope and intercept
        chi2 = sum((y - m*x+c)**2 for x,y in zip(self.x,self.y))
        return chi2

# <codecell>

chi2 = StraightLineChi2(x,y)
describe(chi2)

# <codecell>

m = Minuit(chi2)
m.migrad()
print m.values

# <markdowncell>

# ###Faking a function signature
# This is missing from PyMinuit. iminuit allows you to take funciton sinature by using `func_code.co_varnames` and `func_code.co_argcount`. This is very useful for making a higher order function that takes PDF and data in to calculate appropriate cost function.

# <codecell>

%pylab inline
from iminuit import Minuit, describe, Struct

# <codecell>

#this is very useful if you want to build a generic cost functor
#this is actually how probfit is implemented
from iminuit.util import make_func_code
x = [1,2,3,4,5]
y = [2,4,6,8,10]# y=2x
class Chi2Functor:
    def __init__(self,f,x,y):
        self.f = f
        self.x = x
        self.y = y
        f_sig = describe(f)
        #this is how you fake function 
        #signature dynamically
        self.func_code = make_func_code(f_sig[1:])#docking off independent variable
        self.func_defaults = None #this keeps np.vectorize happy
    def __call__(self,*arg):
        #notice that it accept variable length
        #positional arguments
        chi2 = sum((y-self.f(x,*arg))**2 for x,y in zip(self.x,self.y))
        return chi2

# <codecell>

def linear(x,m,c):
    return m*x+c

def parabola(x,a,b,c):
    return a*x**2 + b*x + c 

# <codecell>

linear_chi2 = Chi2Functor(linear,x,y)
describe(linear_chi2)

# <codecell>

m = Minuit(linear_chi2)
m.migrad();
print m.values

# <codecell>

#now here is the beauty
#you can use the same Chi2Functor now for parabola
parab_chi2 = Chi2Functor(parabola,x,y)
describe(parab_chi2)

# <codecell>

m = Minuit(parab_chi2,x,y)
m.migrad()
print m.values

# <markdowncell>

# ####Last Resort: Forcing function signature
# 
# built-in function normally do not have signature. Function from swig belongs in this categories. Python intro spection will fails and we have to force function signature.

# <codecell>

%pylab inline
from iminuit import Minuit, describe, Struct

# <codecell>

%%cython
#sometimes you get a function with absolutely no signature
def nosig_f(x,y):
    return x**2+(y-4)**2

# <codecell>

#something from swig will give you a function with no
#signature information
try:
    describe(nosig_f)#it raise error
except Exception as e:
    print e

# <codecell>

#Use forced_parameters
m = Minuit(nosig_f, forced_parameters=('x','y'))

# <codecell>

m.migrad()
print m.values

# <codecell>

print f

# <markdowncell>

# ###Frontend
# Frontend affects how the output from migrad/minos etc are displayed. iminuit is shipped with two frontends. ConsoleFrontend print in text format and HtmlFrontend print html object to Ipython notebook. When you construct Minuit object the front end is selected automatically. If you are in IPython it will use Html frontend; otherwise, it will use console fronend. You can force Minuit to use frontend of your choice too.

# <codecell>

%pylab inline
from iminuit import Minuit, describe, Struct

# <codecell>

#this is just showing off console frontend (you can force it)
from iminuit.ConsoleFrontend import ConsoleFrontend
def f(x,y,z):
    return (x-1.)**2 + (y-2.)**2 + (z-3.)**2 -1.
m = Minuit(f, frontend=ConsoleFrontend())

# <codecell>

m.migrad();

# <codecell>





#cython: embedsignature=True
__all__ = ['Minuit']
from libcpp.vector cimport vector
from libcpp.string cimport string
from cpython cimport exc
#from libcpp import bool
from util import *
from warnings import warn
from cython.operator cimport dereference as deref
from libc.math cimport sqrt
from pprint import pprint
from ConsoleFrontend import ConsoleFrontend
from iminuit_warnings import *
from latex import LatexFactory
import _plotting
include "Lcg_Minuit.pxi"
include "Minuit2Struct.pxi"
import array
#our wrapper
cdef extern from "PythonFCN.h":
    #int raise_py_err()#this is very important we need custom error handler
    FunctionMinimum* call_mnapplication_wrapper(\
        MnApplication app,unsigned int i, double tol) except +
    cdef cppclass PythonFCN(FCNBase):
        PythonFCN(\
            object fcn, double up_parm, vector[string] pname,bint thrownan)
        double call "operator()" (vector[double] x) except +#raise_py_err
        double up()
        int getNumCall()
        void set_up(double up)
        void resetNumCall()


#look up map with default
cdef maplookup(m,k,d):
    return m[k] if k in m else d


cdef class Minuit:
    #standard stuff

    cdef readonly object fcn #:fcn
    #cdef readonly object varname #:variable names
    """this should work"""
    cdef readonly object pos2var#:map variable position to varname
    """or this should work"""
    cdef readonly object var2pos#:map varname to position

    #Initial settings
    cdef object initialvalue #:hold initial values
    cdef object initialerror #:hold initial errors
    cdef object initiallimit #:hold initial limits
    cdef object initialfix #:hold initial fix state

    #C++ object state
    cdef PythonFCN* pyfcn #:FCN
    cdef MnApplication* minimizer #:migrad
    cdef FunctionMinimum* cfmin #:last migrad result
    #:last parameter state(from hesse/migrad)
    cdef MnUserParameterState* last_upst

    #PyMinuit compatible field
    cdef public double errordef #:UP parameter
    cdef public double tol #:tolerance migrad stops when edm<0.0001*tol*UP
    cdef public unsigned int strategy #:0 fast 1 default 2 slow but accurate
    #: 0: quiet 1: print stuff the end 2: 1+fit status during call
    #: yes I know the case is wrong but this is to keep it compatible with
    #: PyMinuit
    cdef public print_level
    #: raise runtime error if function evaluate to nan
    cdef readonly bint throw_nan

    #PyMinuit Compatible interface
    cdef readonly object parameters#:tuple of parameter name(correct order)
    cdef readonly object args#:tuple of values
    cdef readonly object values#:map varname -> value
    cdef readonly object errors#:map varname -> parabolic error
    cdef readonly object covariance#:map (v1,v2)->covariance
    cdef readonly double fval#:last value of fcn
    cdef readonly int ncalls#:number of fcn call of last migrad/minos/hesse
    cdef readonly double edm#Estimate distance to minimum
    #:minos error ni a funny map from
    #:(vname,direction)->error
    #:direction is 1.0 for positive error and -1.0 for negative error
    cdef readonly object merrors
    #:global correlation coefficient
    cdef readonly object gcc
    #and some extra
    #:map of
    #:varname -> value
    #:error_varname -> error
    #:limit_varname -> limit
    #:fix_varname -> True/False
    #:user can just use python keyword expansion to use all the argument like
    #:Minuit(fcn,**fitargs)
    cdef public object fitarg
    cdef readonly object narg#: number of arguments
    #: map vname-> struct with various minos error calcumation information
    cdef public object merrors_struct
    cdef public object frontend

    def __init__(self, fcn,
            throw_nan=False, pedantic=True,
            frontend=None, forced_parameters=None, print_level=1,
            errordef=None, **kwds):
        """
        Construct minuit object from given *fcn*

        **Arguments:**

            - **fcn**: function to optimized. Minuit automagically find how to
              call the function and each parameters. More information about how
              Minuit detects function signature can be found in
              :ref:`function-sig-label`

        **Builtin Keyword Arguments:**

            - **throw_nan**: fcn can be set to raise RuntimeError when it
              encounters *nan*. (Default False)

            - **pedantic**: warns about parameters that do not have initial
              value or initial error/stepsize set.

            - **frontend**: Minuit frontend. There are two builtin frontends.

                1. ConsoleFrontend which is design to print out to terminal.

                2. HtmlFrontend which is designed to give a nice output in
                   IPython notebook session.

              By Default, Minuit switch to HtmlFrontend automatically if it
              is called in IPython session. It uses ConsoleFrontend otherwise.

            - **forced_parameters**: tell Minuit not to do function signature
              detection and use this argument instead. (Default None
              (automagically detect signature)

            - **print_level**: set the print_level for this Minuit. 0 is quiet.
              1 print out at the end of migrad/hesse/minos. The reason it
              has this cAmEl case is to keep it compatible with PyMinuit.

            - **errordef**: Optionals. Amount of increase in fcn to be defined
              as 1 :math:`\sigma`. If None is given, it will look at
              `fcn.default_errordef()`. If `fcn.default_errordef()` is not 
              defined or
              not callable iminuit will give a warning and set errordef to 1.
              Default None(which means errordef=1 with a warning).

        **Parameter Keyword Arguments:**

            Similar to PyMinuit. iminuit allows user to set initial value,
            initial stepsize/error, limits of parameters and whether
            parameter should be fixed or not by passing keyword arguments to
            Minuit. This is best explained through an example::

                def f(x,y):
                    return (x-2)**2 + (y-3)**2

            * Initial value(varname)::

                #initial value for x and y
                m = Minuit(f, x=1, y=2)

            * Initial step size/error(fix_varname)::

                #initial step size for x and y
                m = Minuit(f, error_x=0.5, error_y=0.5)

            * Limits (limit_varname=tuple)::

                #limits x and y
                m = Minuit(f, limit_x=(-10,10), limit_y=(-20,20))

            * Fixing parameters::

                #fix x but vary y
                m = Minuit(f, fix_x=True)

            .. note::

                Tips: you can use python dictionary expansion to
                programatically change fitting argument.

                ::

                    kwdarg = dict(x=1., error_x=0.5)
                    m = Minuit(f, **kwdarg)

                You can obtain also obtain fit arguments from Minuit object
                to reuse it later too. Note that fitarg will be automatically
                updated to minimum value and corresponding error when you ran
                migrad/hesse::

                    m = Minuit(f, x=1, error_x=0.5)
                    my_fitarg = m.fitarg
                    another_fit = Minuit(f, **my_fitarg)

        """

        args = describe(fcn) if forced_parameters is None\
               else forced_parameters
        self._check_extra_args(args,kwds)
        narg = len(args)
        self.fcn = fcn

        self.frontend = self._auto_frontend() if frontend is None else frontend

        #maintain 2 dictionary 1 is position to varname
        #and varname to position
        #self.varname = args
        self.pos2var = {i: k for i, k in enumerate(args)}
        self.var2pos = {k: i for i, k in enumerate(args)}

        self.args, self.values, self.errors = None, None, None

        self.initialvalue = {x:maplookup(kwds,x,0.) for x in args}
        self.initialerror = \
            {x:maplookup(kwds,'error_'+x,1.) for x in args}
        self.initiallimit = \
            {x:maplookup(kwds,'limit_'+x,None) for x in args}
        self.initialfix = \
            {x:maplookup(kwds,'fix_'+x,False) for x in args}

        self.pyfcn = NULL
        self.minimizer = NULL
        self.cfmin = NULL
        self.last_upst = NULL

        if errordef is None:
            default_errordef = getattr(fcn,'default_errordef', None)
            if not callable(default_errordef):
                if pedantic:
                    warn(InitialParamWarning(
                        'errordef is not given. Default to 1.'))
                self.errordef = 1.0
            else:
                self.errordef = default_errordef()
        else:
            self.errordef = errordef
        self.tol = 0.1
        self.strategy = 1
        self.print_level = print_level
        set_migrad_print_level(print_level)
        self.throw_nan = throw_nan

        self.parameters = args
        self.args = tuple(self.initialvalue[k] for k in args)
        self.values = {k:self.initialvalue[k] for k in args}
        self.errors = {k:self.initialerror[k] for k in args}
        self.covariance = None
        self.fval = 0.
        self.ncalls = 0
        self.edm = 1.
        self.merrors = {}
        self.gcc = None
        if pedantic: self.pedantic(kwds)

        self.fitarg = {}
        self.fitarg.update(self.initialvalue)
        self.fitarg.update(
            {'error_'+k:v for k,v in self.initialerror.items()})
        self.fitarg.update(
            {'limit_'+k:v for k,v in self.initiallimit.items()})
        self.fitarg.update(
            {'fix_'+k:v for k,v in self.initialfix.items()})

        self.narg = len(self.parameters)

        self.merrors_struct = {}


    def migrad(self, int ncall=10000, resume=True, int nsplit=1):
        """Run migrad.

        Migrad is an age-tested(over 40 years old, no kidding), super
        robust and stable minimization algorithm. It even has
        `wiki page <http://en.wikipedia.org/wiki/MINUIT>`_.
        You can read how it does the magic at
        `here <http://wwwasdoc.web.cern.ch/wwwasdoc/minuit/minmain.html>`_.

        **Arguments:**

            * **ncall**: integer (approximate) maximum number of call before
              migrad stop trying. Default 10000

            * **resume**: boolean indicating whether migrad should resume from
              previous minimizer attempt(True) or should start from the
              beginning(False). Default True.

            * **split**: split migrad in to *split* runs. Max fcn call
              for each run is ncall/nsplit. Migrad stops when it found the
              function minimum to be valid or ncall is reached. This is useful
              for getting progress. However, you need to make sure that
              ncall/nsplit is large enough. Otherwise, migrad will think
              that the minimum is invalid due to exceeding max call
              (ncall/nsplit). Default 1(no split).

        **Return:**

            :ref:`function-minimum-sruct`, list of :ref:`minuit-param-struct`
        """
        #construct new fcn and migrad if
        #it's a clean state or resume=False
        cdef MnUserParameterState* ups = NULL
        cdef MnStrategy* strat = NULL

        if self.print_level>0: self.frontend.print_banner('MIGRAD')

        if not resume or self.is_clean_state():
            self.construct_FCN()
            if self.minimizer is not NULL: del self.minimizer
            ups = self.initialParameterState()
            strat = new MnStrategy(self.strategy)
            self.minimizer = \
                    new MnMigrad(deref(self.pyfcn),deref(ups),deref(strat))
            del ups; ups=NULL
            del strat; strat=NULL

        if not resume: self.pyfcn.resetNumCall()

        del self.cfmin #remove the old one

        #this returns a real object need to copy
        ncall_round = round(1.0*(ncall)/nsplit)
        assert(ncall_round>0)
        totalcalls = 0
        first = True
        while (first) or \
                (not self.cfmin.isValid() and totalcalls < ncall):
            first=False
            self.cfmin = call_mnapplication_wrapper(
                    deref(self.minimizer),ncall_round,self.tol)
            del self.last_upst
            self.last_upst = new MnUserParameterState(self.cfmin.userState())
            totalcalls+=ncall_round#self.cfmin.nfcn()
            if self.print_level>1 and nsplit!=1: self.print_fmin()

        del self.last_upst
        self.last_upst = new MnUserParameterState(self.cfmin.userState())
        self.refreshInternalState()

        if self.print_level>0: self.print_fmin()

        return self.get_fmin(), self.get_param_states()


    def hesse(self):
        """Run HESSE.

        HESSE estimates error matrix by the `second derivative at the minimim
        <http://en.wikipedia.org/wiki/Hessian_matrix>`_. This error matrix
        is good if your :math:`\chi^2` or likelihood profile is parabolic at
        the minimum. From my experience, most of simple fits are.

        :meth:`minos` makes no parabolic assumption and scan the likelihood
        and give the correct error asymmetric error in all cases(Unless your
        likelihood profile is utterly discontinuous near the minimum). But
        it is much more computationally expensive.

        **Returns**
            list of :ref:`minuit-param-struct`
        """

        cdef MnHesse* hesse = NULL
        cdef MnUserParameterState upst
        if self.print_level>0: self.frontend.print_banner('HESSE')
        if self.cfmin is NULL:
            raise RuntimeError('Run migrad first')
        hesse = new MnHesse(self.strategy)
        upst = hesse.call(deref(self.pyfcn),self.cfmin.userState())
        if not upst.hasCovariance():
            warn("HESSE Failed. Covariance and GlobalCC will not be available",
                HesseFailedWarning)
        del self.last_upst
        self.last_upst = new MnUserParameterState(upst)
        self.refreshInternalState()
        del hesse

        if self.print_level>0:
            self.print_param()
            self.print_matrix()

        return self.get_param_states()


    def minos(self, var = None, sigma = 1., unsigned int maxcall=1000):
        """Run minos for parameter *var*

        If *var* is None it runs minos for all parameters

        **Arguments:**

            - **var**: optional variable name. Default None.(run minos for
              every variable)
            - **sigma**: number of :math:`\sigma` error. Default 1.0.

        **Returns**

            Dictionary of varname to :ref:`minos-error-struct`
            if minos is requested for all parameters.

        """
        if self.pyfcn is NULL or self.cfmin is NULL:
            raise RuntimeError('Minos require function to be at the minimum.'
                               ' Run migrad first.')
        cdef unsigned int index = 0
        cdef MnMinos* minos = NULL
        cdef MinosError mnerror
        cdef char* name = NULL
        cdef double oldup = self.pyfcn.up()
        self.pyfcn.set_up(oldup*sigma*sigma)
        if self.print_level>0: self.frontend.print_banner('MINOS')
        if not self.cfmin.isValid():
            raise RuntimeError(('Function mimimum is not valid. Make sure'
                ' migrad converge first'))
        if var is not None and var not in self.parameters:
                raise RuntimeError('Specified parameters(%r) cannot be found'
                    ' in parameter list :'%var+str(self.parameters))

        varlist = [var] if var is not None else self.parameters

        fixed_param = self.list_of_fixed_param()
        for vname in varlist:
            index = self.cfmin.userState().index(vname)

            if vname in fixed_param:
                if var is not None:#specifying vname but it's fixed
                    warnings.warn(RuntimeWarning(
                        'Specified variable name for minos is set to fixed'))
                    return None
                continue
            minos = new MnMinos(deref(
                self.pyfcn), deref(self.cfmin), self.strategy)
            mnerror = minos.minos(index,maxcall)
            self.merrors_struct[vname]=minoserror2struct(mnerror)
            if self.print_level>0:
                self.frontend.print_merror(
                    vname, self.merrors_struct[vname])
        self.refreshInternalState()
        del minos
        self.pyfcn.set_up(oldup)
        return self.merrors_struct


    def matrix(self, correlation=False, skip_fixed=True):
        """return error/correlation matrix in tuple or tuple format."""
        if self.last_upst is NULL:
            raise RuntimeError("Run migrad/hesse first")
        if not skip_fixed:
            raise RunTimeError('skip_fixed=False is not supported')
        if not self.last_upst.hasCovariance():
            raise RuntimeError(
                "Covariance is not valid. May be the last Hesse call failed?")

        cdef MnUserCovariance cov = self.last_upst.covariance()
        params = self.list_of_vary_param()
        if correlation:
            ret = tuple(
                tuple(cov.get(iv1,iv2)/sqrt(cov.get(iv1,iv1)*cov.get(iv2,iv2))
                    for iv1,v1 in enumerate(params))\
                    for iv2,v2 in enumerate(params)
                )
        else:
            ret = tuple(
                tuple(cov.get(iv1,iv2)
                    for iv1,v1 in enumerate(params))\
                    for iv2,v2 in enumerate(params)
                )
        return ret


    def print_matrix(self, **kwds):
        """show error_matrix"""
        matrix = self.matrix(correlation=True)
        vnames = self.list_of_vary_param()
        self.frontend.print_matrix(vnames, matrix, **kwds)


    def latex_matrix(self):
        """Build :class:`LatexFactory` object that contains correlation 
        matrix
        """
        matrix = self.matrix(correlation=True)
        vnames = self.list_of_vary_param()
        return LatexFactory.build_matrix(vnames, matrix)


    def np_matrix(self, correlation=False):
        """return error/correlation matrix in numpy array format."""
        import numpy as np
        #TODO make a not so lazy one
        return np.array(matrix)


    def is_fixed(self,vname):
        """check if variable *vname* is (initialy) fixed"""
        if vname not in self.parameters:
            raise RuntimeError('Cannot find %s in list of variables.')
        cdef unsigned int index = self.var2pos[vname]
        if self.last_upst is NULL:
            return self.initialfix[vname]
        else:
            return self.last_upst.minuitParameters()[index].isFixed()

    def _prepare_param(self):
        cdef vector[MinuitParameter] vmps = self.last_upst.minuitParameters()
        cdef int i
        tmp = []
        for i in range(vmps.size()):
            tmp.append(minuitparam2struct(vmps[i]))
        return tmp

    #dealing with frontend conversion
    def print_param(self, **kwds):
        """print current parameter state. Extra keyword arguments will
        be passed to frontend.print_param"""
        if self.last_upst is NULL:
            self.print_initial_param(**kwds)
            return
        p = self._prepare_param()
        self.frontend.print_param(p, self.merrors_struct, **kwds)


    def latex_param(self):
        """build :class:`iminuit.latex.LatexTable` for current
        parameter"""
        p = self._prepare_param()
        return LatexFactory.build_param_table(p, self.merrors_struct)


    def _prepare_initial_param(self):
        tmp = []
        for i,vname in enumerate(self.parameters):
            mps = Struct(
            number = i+1,
            name = vname,
            value = self.initialvalue[vname],
            error = self.initialerror[vname],
            is_const = False,
            is_fixed = self.initialfix[vname],
            has_limits = self.initiallimit[vname] is not None,
            lower_limit = self.initiallimit[vname][0]
                if self.initiallimit[vname] is not None else -999,
            upper_limit = self.initiallimit[vname][1]
                if self.initiallimit[vname] is not None else 999,
            has_lower_limit = self.initiallimit[vname] is not None,
            has_upper_limit = self.initiallimit[vname] is not None
            )
            tmp.append(mps)
        return tmp


    def print_initial_param(self, **kwds):
        """Print initial parameters"""
        p = self._prepare_initial_param()
        self.frontend.print_param(p, {}, **kwds)


    def latex_initial_param(self):
        """build :class:`iminuit.latex.LatexTable` for initial
        parameter"""
        p = self._prepare_initial_param()
        return LatexFactory.build_param_table(p, {})


    def print_fmin(self):
        """print current function minimum state"""
        #cdef MnUserParameterState ust = MnUserParameterState(
        #                               self.cfmin.userState())
        if self.cfmin is NULL:
            raise RuntimeError("Function minimum has not been calculated.")
        sfmin = cfmin2struct(self.cfmin)
        ncalls = 0 if self.pyfcn is NULL else self.pyfcn.getNumCall()

        self.frontend.print_hline()
        self.frontend.print_fmin(sfmin,self.tol,ncalls)
        self.print_param()
        self.frontend.print_hline()



    def print_all_minos(self):
        """print all minos errors(and its states)"""
        for vname in self.list_of_vary_param():
            if vname in self.merrors_struct:
                self.frontend.print_merror(vname, self.merrors_struct[vname])


    def set_up(self, double errordef):
        """
        alias for :meth:`set_errordef`
        """
        self.set_errordef(errordef)


    def set_errordef(self, double errordef):
        """
        set error parameter 1 for :math:`\chi^2` and 0.5 for log likelihood

        .. seealso::
            http://wwwasdoc.web.cern.ch/wwwasdoc/minuit/node31.html

        """
        self.errordef = errordef
        if self.pyfcn is not NULL:
            self.pyfcn.set_up(errordef)


    def set_strategy(self,stra):
        """set strategy 0=fast , 1=default, 2=slow but accurate"""
        self.strategy=stra


    def set_print_level(self, lvl):
        """set printlevel 0 quiet, 1 normal, 2 paranoid, 3 really paranoid """
        self.print_level = lvl
        set_migrad_print_level(lvl)


    def get_fmin(self):
        """return current FunctionMinimum Struct"""
        return cfmin2struct(self.cfmin) if self.cfmin is not NULL else None


    #expose internal state using various struct
    def get_param_states(self):
        """Return a list of current MinuitParameter Struct
        for all parameters
        """
        if self.last_upst is NULL:
            return self.get_initial_param_state()
        cdef vector[MinuitParameter] vmps = self.last_upst.minuitParameters()
        cdef int i
        ret = []
        for i in range(vmps.size()):
            ret.append(minuitparam2struct(vmps[i]))
        return ret


    def get_merrors(self):
        """Returns a dictionary of varname-> MinosError Struct"""
        return self.merrors_struct


    def get_initial_param_state(self):
        """get initiail setting inform of MinuitParameter Struct"""
        raise NotImplementedError


    def get_num_call_fcn(self):
        """return number of total call to fcn(not just the last operation)"""
        return 0 if self.pyfcn is NULL else self.pyfcn.getNumCall()


    def migrad_ok(self):
        """check if minimum is valid"""
        return self.cfmin is not NULL and self.cfmin.isValid()


    def matrix_accurate(self):
        """check if covariance(of the last migrad) is accurate."""
        return self.last_upst is not NULL and\
               self.cfmin is not NULL and\
               self.cfmin.hasAccurateCovar()


    def list_of_fixed_param(self):
        """return list of (initially) fixed parameters"""
        return [v for v in self.parameters if self.initialfix[v]]


    def list_of_vary_param(self):
        """return list of (initially) float vary parameters"""
        return [v for v in self.parameters if not self.initialfix[v]]


    #Various utility functions
    cdef construct_FCN(self):
        """(re)construct FCN"""
        del self.pyfcn
        self.pyfcn = new PythonFCN(
                self.fcn,
                self.errordef,
                self.parameters,
                self.throw_nan)


    def is_clean_state(self):
        """check if minuit is at clean state ie. no migrad call"""
        return self.pyfcn is NULL and \
            self.minimizer is NULL and self.cfmin is NULL


    cdef void clear_cobj(self):
        #clear C++ internal state
        del self.pyfcn;self.pyfcn = NULL
        del self.minimizer;self.minimizer = NULL
        del self.cfmin;self.cfmin = NULL
        del self.last_upst;self.last_upst = NULL


    def __dealloc__(self):
        self.clear_cobj()


    def pedantic(self, kwds):
        for vn in self.parameters:
            if vn not in kwds:
                warn(('Parameter %s does not have initial value. '
                    'Assume 0.') % (vn), InitialParamWarning)
            if 'error_'+vn not in kwds and 'fix_'+param_name(vn) not in kwds:
                warn(('Parameter %s is floating but does not '
                    'have initial step size. Assume 1.') % (vn),
                    InitialParamWarning)
        for vlim in extract_limit(kwds):
            if param_name(vlim) not in self.parameters:
                warn(('%s is given. But there is no parameter %s. '
                    'Ignore.' % (vlim, param_name(vlim)), InitialParamWarning))
        for vfix in extract_fix(kwds):
            if param_name(vfix) not in self.parameters:
                warn(('%s is given. But there is no parameter %s. \
                    Ignore.' % (vfix, param_name(vfix)), InitialParamWarning))
        for verr in extract_error(kwds):
            if param_name(verr) not in self.parameters:
                warn(('%s float. But there is no parameter %s. \
                    Ignore.') % (verr, param_name(verr)), InitialParamWarning)


    def mnprofile(self, vname, bins=30, bound=2,
            subtract_min=False):
        """calculate minos profile around specify range ie. Migrad results
        with **vname** fixed at various places within **bound**

        **Arguments**

            * **vname** variable name to scan

            * **bins** number of scanning bin. Default 30.

            * **bound**
                If bound is tuple, (left, right) scanning bound.
                If bound is a number, it specifies how many :math:`\sigma`
                symmetrically from minimum (minimum+- bound* :math:`\sigma`).
                Default 2

            * **subtract_min** subtract_minimum off from return value. This
                makes it easy to label confidence interval. Default False.

        **Returns**

            bins(center point), value, migrad results
        """
        if vname not in self.parameters:
            raise ValueError('Unknown parameter %s'%vname)

        if isinstance(bound, (int,long,float)):
            if not self.matrix_accurate():
                warn('Specify nsigma bound but error '
                     'but error matrix is not accurate.')
            start = self.values[vname]
            sigma = self.errors[vname]
            bound = (start -  bound*sigma, start + bound*sigma)
        blength = bound[1]-bound[0]
        binstep = blength/(bins-1)
        
        values = array.array('d',(bound[0]+binstep*i for i in xrange(bins)))
        results = array.array('d')
        migrad_status = []
        for i, v in enumerate(values):
            fitparam = self.fitarg.copy()
            fitparam[vname] = v
            fitparam['fix_%s'%vname] = True
            m = Minuit(self.fcn, print_level=0,
                    pedantic=False, forced_parameters=self.parameters,
                    **fitparam)
            m.migrad()
            migrad_status.append(m.migrad_ok())
            if not m.migrad_ok():
                warn(('Migrad fails to converge for %s=%f')%(vname, v))
            results.append(m.fval)

        if subtract_min:
            themin = min(results)
            results = array.array('d',(x-themin for x in results))

        return values, results, migrad_status


    def draw_mnprofile(self, vname, bins=30, bound=2, subtract_min=False,
        band=True, text=True):
        """Draw minos profile around specify range ie. Migrad results
        with **vname** fixed at various places within **bound**

        **Arguments**

            * **vname** variable name to scan

            * **bins** number of scanning bin. Default 30.

            * **bound**
                If bound is tuple, (left, right) scanning bound.
                If bound is a number, it specifies how many :math:`\sigma`
                symmetrically from minimum (minimum+- bound* :math:`\sigma`).
                Default 2

            * **subtract_min** subtract_minimum off from return value. This
                makes it easy to label confidence interval. Default False.

            * **band** show green band to indicate the increase of fcn by
              *errordef*. Default True.

            * **text** show text for the location where the fcn is increased
              by *errordef*. This is less accurate than :meth:`minos`.
              Default True.

        **Returns**

            bins(center point), value, migrad results

        .. plot:: pyplots/draw_mnprofile.py
            :include-source:

        """
        x, y, s= self.mnprofile(vname, bins, bound, subtract_min)
        return _plotting.draw_profile(self, vname, x, y, s, 
                                      band=band, text=text)


    def profile(self, vname, bins=100, bound=2, args=None, subtract_min=False):
        """calculate cost function profile around specify range.

        **Arguments**

            * **vname** variable name to scan

            * **bins** number of scanning bin. Default 100.

            * **bound**
              If bound is tuple, (left, right) scanning bound.
              If bound is a number, it specifies how many :math:`\sigma`
              symmetrically from minimum (minimum+- bound* :math:`\sigma`).
              Default 2

            * **subtract_min** subtract_minimum off from return value. This
                makes it easy to label confidence interval. Default False.

        **Returns**

            bins(center point), value

        .. seealso::

            :meth:`mnprofile`

        """
        if isinstance(bound, (int,long,float)):
            start = self.values[vname]
            sigma = self.errors[vname]
            bound = (start-bound*sigma,
                    start+bound*sigma)
        blength = bound[1]-bound[0]
        binstep = blength/(bins-1.)
        args = list(self.args) if args is None else args
        #center value
        bins = array.array('d',(bound[0]+binstep*i for i in xrange(bins)))
        ret = array.array('d')
        pos = self.var2pos[vname]
        if subtract_min and self.cfmin is NULL:
            raise RunTimeError("Request for minimization "
                "subtraction but no minimization has been done. "
                "Run migrad first.")
        minval = self.cfmin.fval() if subtract_min else 0.
        for val in bins:
            args[pos] = val
            ret.append(self.fcn(*args)-minval)
        return bins, ret


    def draw_profile(self, vname, bins=100, bound=2, args=None,
        subtract_min=False, band=True, text=True):
        """
        A convenient wrapper for drawing profile using matplotlib

        .. note::
            This is not a real minos profile. It's just a simple 1D scan.
            The number shown on the plot is taken from the green band.
            They are not minos error. To get a real minos profile call
            :meth:`mnprofile` or :meth:`draw_mnprofile`

        **Arguments**

            In addition to argument listed on :meth:`profile`. draw_profile
            take these addition argument:

            * **band** show green band to indicate the increase of fcn by
              *errordef*. Note again that this is NOT minos error in general.
              Default True.

            * **text** show text for the location where the fcn is increased
              by *errordef*. This is less accurate than :meth:`minos`
              Note again that this is NOT minos error in general. Default True.

        .. seealso::
            :meth:`mnprofile`
            :meth:`draw_mnprofile`
            :meth:`profile`
        """
        x, y = self.profile(vname, bins, bound, args, subtract_min)
        x, y, s = _plotting.draw_profile(self, vname, x, y,
                                         band=band, text=text)
        return x, y

    def contour(self, x, y, bins=20, bound=2, args=None, subtract_min=False):
        """2D countour scan.

        The contour returns is obtained by fixing all others parameters and
        varying **x** and **y**.

        **Arguments**

            - **x** variable name for X axis of scan

            - **y** variable name for Y axis of scan

            - **bound**
                If bound is 2x2 array [[v1min,v1max],[v2min,v2max]].
                If bound is a number, it specifies how many :math:`\sigma`
                symmetrically from minimum (minimum+- bound*:math:`\sigma`).
                Default 2

            - **subtract_min** subtract_minimum off from return value. This
                makes it easy to label confidence interval. Default False.

        **Returns**

            x_bins, y_bins, values

            values[y, x] <-- this choice is so that you can pass it
            to through matplotlib contour()

        .. seealso::

            :meth:`mncontour`

        .. note::

            If `subtract_min=True`, the return value has the minimum subtracted
            off. The value on the contour can be interpreted *loosely* as
            :math:`i^2 \\times \\textrm{up}` where i is number of standard
            deviation away from the fitted value *WITHOUT* taking into account
            correlation with other parameters that's fixed.

        """
        #don't want to use numpy as requirement for this
        if isinstance(bound, (int,long,float)):
            x_start = self.values[x]
            x_sigma = self.errors[x]
            x_bound = (x_start-bound*x_sigma, x_start+bound*x_sigma)
            y_start = self.values[y]
            y_sigma = self.errors[y]
            y_bound = (y_start-bound*y_sigma, y_start+bound*y_sigma)
        else:
            x_bound = bound[0]
            y_bound = bound[1]
        
        x_bins = bins
        y_bins = bins

        x_blength = x_bound[1]-x_bound[0]
        x_binstep = x_blength/(x_bins-1.)

        y_blength = y_bound[1]-y_bound[0]
        y_binstep = y_blength/(y_bins-1.)

        x_val = array.array('d',(x_bound[0]+x_binstep*i for i in xrange(x_bins)))
        y_val = array.array('d',(y_bound[0]+y_binstep*i for i in xrange(y_bins)))

        x_pos = self.var2pos[x]
        y_pos = self.var2pos[y]

        args = list(self.args) if args is None else args

        if subtract_min and self.cfmin is NULL:
            raise RunTimeError("Request for minimization "
                "subtraction but no minimization has been done. "
                "Run migrad first.")
        minval = self.cfmin.fval() if subtract_min else 0.

        ret = list()
        for yy in y_val:
            args[y_pos] = yy
            tmp = array.array('d')
            for xx in x_val:
                args[x_pos] = xx
                tmp.append(self.fcn(*args)-minval)
            ret.append(tmp)

        return x_val, y_val, ret


    def mncontour(self, x, y, int numpoints=20, sigma=1.0):
        """
        Minos contour scan. A proper n **sigma** contour scan. This is line
        where the minimum of fcn  with x,y is fixed at points on the line and
        letting the rest of variable varied is change by **sigma** * errordef^2
        . The calculation is very very expensive since it has to run migrad
        at various points.

        .. note::
            See http://wwwasdoc.web.cern.ch/wwwasdoc/minuit/node7.html

        **Arguments**

            - **x** string variable name of the first parameter

            - **y** string variable name of the second parameter

            - **numpoints** number of points on the line to find. Default 20.

            - **sigma** number of sigma for the contour line. Default 1.0.

        **Returns**

            x minos error struct, y minos error struct, contour line

            contour line is a list of the form
            [[x1,y1]...[xn,yn]]

        """
        if self.pyfcn is NULL or self.cfmin is NULL:
            raise ValueError('Run Migrad first')

        cdef unsigned int ix = self.var2pos[x]
        cdef unsigned int iy = self.var2pos[y]

        vary_param = self.list_of_vary_param()

        if x not in vary_param or y not in vary_param:
            raise ValueError('mncontour has to be run on vary parameters.')

        cdef double oldup = self.pyfcn.up()
        self.pyfcn.set_up(oldup*sigma*sigma)

        cdef auto_ptr[MnContours] mnc = auto_ptr[MnContours](
                                            new MnContours(deref(self.pyfcn),
                                                    deref(self.cfmin),
                                                    self.strategy))

        cdef ContoursError cerr = mnc.get().contour(ix, iy, numpoints)

        xminos = minoserror2struct(cerr.xMinosError())
        yminos = minoserror2struct(cerr.yMinosError())

        self.pyfcn.set_up(oldup)

        return xminos, yminos, cerr.points() #using type coersion here


    def mncontour_grid(self, x, y, bins=100, nsigma=2, numpoints=20,
                        int sigma_res=4, edges=False):
        """
        compute gridded minos contour.

        **Arguments**

            - **x**,**y** parameter name

            - **bins** number of bins in the grid. The boundary of the grid is
              selected automatically by the minos error computed. Default 100.

            - **nsigma** number of sigma to draw. Default 2

            - **numpoints** number of points to calculate mncontour for each
              sigma points(there are sigma_res*nsigma total)

            - **sigma_res** number of sigma level to calculate MnContours

            - **edges** Return bin edges instead of mid value(pass True if you
              want to draw it using pcolormesh)

        **Returns**

            xgrid, ygrid, sigma, rawdata

            rawdata is tuple of (x,y,sigma_level)

        .. seealso::

            :meth:`draw_mncontour`

        .. plot:: pyplots/draw_mncontour.py
            :include-source:

        """
        return _plotting.mncontour_grid(self, x, y, numpoints,
                                        nsigma, sigma_res, bins, edges)


    def draw_mncontour(self, x, y, bins=100, nsigma=2,
                        numpoints=20, sigma_res=4):
        """
        draw minos contour

        **Arguments**

            - **x**, **y** parameter name

            - **bins** number of bin in contour grid.

            - **nsigma** number of sigma contour to draw

            - **numpoints** number of points to calculate for each contour

            - **sigma_res** number of sigma level to calculate MnContours.
              Default 4.

        **returns**

            x, y, gridvalue, contour

            gridvalue is interorlated nsigma
        """
        return _plotting.draw_mncontour(self, x, y, bins, nsigma,
                                        numpoints, sigma_res)


    def draw_contour(self, x, y, bins=20, bound=2, args=None, 
                     show_sigma=False):
        """
        Convenient wrapper for drawing contour. The argument is the same as
        :meth:`contour`. If `show_sigma=True`(Default), the label on the
        contour lines will show how many :math:`\sigma` away from the optimal
        value instead of raw value.

        .. note::

            Like :meth:`contour`. The error shown on the plot is not strictly
            1 :math:`\sigma` contour since the other parameters are fixed.

        .. seealso::

            :meth:`contour`
            :meth:`mncontour`

        """
        return _plotting.draw_contour(self, x, y, bins, 
                                      bound, args, show_sigma)


    cdef refreshInternalState(self):
        """refresh internal state attributes.
        These attributes should be in a function instead
        but kept here for PyMinuit compatiblity
        """
        cdef vector[MinuitParameter] mpv
        cdef MnUserCovariance cov
        if self.last_upst is not NULL:
            mpv = self.last_upst.minuitParameters()
            self.values = {}
            self.errors = {}
            self.args = []
            for i in range(mpv.size()):
                self.args.append(mpv[i].value())
                self.values[mpv[i].name()] = mpv[i].value()
                self.errors[mpv[i].name()] = mpv[i].error()
            self.args = tuple(self.args)
            self.fitarg.update(self.values)
            self.fitarg.update({'error_'+k:v for k,v in self.errors.items()})
            vary_param = self.list_of_vary_param()
            if self.last_upst.hasCovariance():
                cov = self.last_upst.covariance()
                self.covariance =\
                     {(v1,v2):cov.get(i,j)\
                         for i,v1 in enumerate(vary_param)\
                         for j,v2 in enumerate(vary_param)}
            else:
                self.covariance = None
            self.fval = self.last_upst.fval()
            self.ncalls = self.last_upst.nfcn()
            self.edm = self.last_upst.edm()
            self.gcc = None
            if self.last_upst.hasGlobalCC() and self.last_upst.globalCC().isValid():
                self.gcc = {v:self.last_upst.globalCC().globalCC().at(i)\
                    for i,v in enumerate(self.list_of_vary_param())}
        self.merrors = {(k,1.0):v.upper
                       for k,v in self.merrors_struct.items()}
        self.merrors.update({(k,-1.0):v.lower
                       for k,v in self.merrors_struct.items()})


    cdef MnUserParameterState* initialParameterState(self) except *:
        """construct parameter state from initial array.
        caller is responsible for cleaning up the pointer
        """
        cdef MnUserParameterState* ret = new MnUserParameterState()
        cdef double lb
        cdef double ub
        for v in self.parameters:
            ret.add(v,self.initialvalue[v],self.initialerror[v])
        for v in self.parameters:
            if self.initiallimit[v] is not None:
                lb,ub = self.initiallimit[v]
                if lb >= ub:
                    raise ValueError(
                        'limit for parameter %s is invalid. %r'%(v,(lb,ub)))
                ret.setLimits(v,lb,ub)
        for v in self.parameters:
            if self.initialfix[v]:
                ret.fix(v)
        return ret


    def _auto_frontend(self):
        """determine front end automatically.
        If this session is an IPYTHON session then use Html frontend,
        Console Frontend otherwise.
        """
        try:
            __IPYTHON__
            from HtmlFrontend import HtmlFrontend
            return HtmlFrontend()
        except NameError:
            return ConsoleFrontend()


    def _check_extra_args(self,parameters,kwd):
        """check keyword arguments to find unwanted/typo keyword arguments"""
        fixed_param = set('fix_'+p for p in parameters)
        limit_param = set('limit_'+p for p in parameters)
        error_param = set('error_'+p for p in parameters)
        for k in kwd.keys():
            if k not in parameters and\
                    k not in fixed_param and\
                    k not in limit_param and\
                    k not in error_param:
                raise RuntimeError(
                        ('Cannot understand keyword %s. May be a typo?\n'
                        'The parameters are %r')%(k,parameters))



#!/usr/bin/env python2
import sys, string, os

#----------------------------------------------------------------------------------
def genfile( fname ) :
  s = ''
  command = 'dumpbin/symbols '+ fname 
  for line in os.popen(command).readlines() :
    if line.find('SECT') == -1 : continue 
    words = line.split()
    if 'External' in words :
      symbol = words[words.index('|')+1]
      if symbol.find('??_') != -1 : continue
      if symbol[0:5] == '_SEAL' : symbol = symbol[1:]
      if '()' not in words : s += '  '+ symbol + '\tDATA\n'
      else :                 s += '  '+ symbol + '\n'
  return s
  
#----------------------------------------------------------------------------------
def usage() :
  print 'Usage:'
  print '  gendef [object/library file] [options]'
  print 'Try "gendef --help" for more information.'
  sys.exit()
#----------------------------------------------------------------------------------
def help() :
  print """Generates Module-Definition (.def) File\n
  print 'Usage:'
  print '  gendef [object/library file] [options]'
  Options:
     -h, --help
       Print this help\n
     -o, --output=
       Output file\n 
   """ 
  sys.exit()
#----------------------------------------------------------------------------------
def main() :
  import getopt
  if len(sys.argv) < 2 :
    print 'No object/library file as input'
    usage()
  if sys.argv[1][0] == '-' :
    infile  = None  
    options = sys.argv[1:]
  else :
    infile  = sys.argv[1]
    options = sys.argv[2:]
  output = None;
  #----Process options--------------------------------
  try:
    opts, args = getopt.getopt(options, 'ho:', ['help','output='])
  except getopt.GetoptError:
    usage()
  for o, a in opts:
    if o in ('-h', '--help'):
      help()
    if o in ('-o', '--output'):
      output = a
  #--------------------------------------------------------------------
  if output : 
      fo = open(output,'w')
      library = string.split (output, '.' )[0]
      library = os.path.split(library)[1]
  else      : 
       fo = sys.stdout
       library = 'tmp'
#  library = os.path.split(infile)[1]
  header  = 'LIBRARY '+ library + '\n'
  header += 'EXPORTS'+'\n'
  fo.write(header)
  if os.path.isdir(infile) :
    for f in os.listdir(infile) :
      if f[f.find('.'):] not in ('.obj','.lib') : continue
      fo.write( genfile(infile+'\\'+f) )  
  elif os.path.isfile(infile) :
    fo.write( genfile(infile) )
  

#---------------------------------------------------------------------
if __name__ == "__main__":
  main()




#describe needs its own separate test
from nose.tools import *
from iminuit.util import describe

#simple function
def f(x,y):
    pass

def test_simple():
    assert_equal(describe(f,True), ['x','y'])

#test bound method
class A:
    def f(self,x,y):
        pass

def test_bound():
    a=A()
    assert_equal(describe(a.f,True),['x','y'])

#unbound method
def test_unbound():
    assert_equal(describe(A.f,True),['self','x','y'])

#faking func code
class Func_Code:
    def __init__(self,varname):
        self.co_varnames=varname
        self.co_argcount=len(varname)

#test __call__
class Func1:
    def __init__(self):
        pass
    def __call__(self,x,y):
        return (x-2.)**2 + (y-5.)**2 + 10

def test_call():
    f1 = Func1()
    assert_equal(describe(f1,True),['x','y'])

#fake func
class Func2:
    def __init__(self):
        self.func_code = Func_Code(['x','y'])
    def __call__(self,*arg):
        return (arg[0]-2.)**2 + (arg[1]-5.)**2 + 10

def test_fakefunc():
    f2 = Func2()
    assert_equal(describe(f2,True),['x','y'])

#builtin (parsing doc)
def test_builtin():
    assert_equal(describe(min),['iterable','key'])

#any good way to test cython builtin call?



from nose.tools import *
from iminuit.latex import LatexTable

def test_table():
    ltt = LatexTable(data=[['alpha',10.123],['alpha_s',30]])
    assert_equal(str(ltt),
    '\\begin{tabular}{|c|c|}\n\\hline\n$\\alpha$ &     10.123\\\\\n\\hline\n$\\alpha_{s}$ & 30\\\\\n\\hline\n\\end{tabular}'
    )

def test_table_color():
    ltt = LatexTable(data=[['alpha',10.123],['alpha_s',30]])
    ltt.set_cell_color(0,0,(111,123,95))
    assert_equal(str(ltt),
    '%\\usepackage[table]{xcolor} % include this for color\n%\\usepackage{rotating} % include this for rotate header\n%\\documentclass[xcolor=table]{beamer} % for beamer\n\\begin{tabular}{|c|c|}\n\\hline\n\\cellcolor[RGB]{111,123,95} $\\alpha$ &     10.123\\\\\n\\hline\n$\\alpha_{s}$ & 30\\\\\n\\hline\n\\end{tabular}'
    )

def test_latexmap():
    ltt = LatexTable(data=[['alpha',10.123,20],['alpha_s',30,40]],
                     latex_map={'alpha':'beta'})
    assert_equal(
        ltt._format('alpha'),
        'beta'
        )

def test_smartlatex():
    ltt = LatexTable(data=[['alpha',10.123,20],['alpha_s',30,40]])
    assert_equal(
        ltt._convert_smart_latex('alpha'),
        r'$\alpha$'
        )
    assert_equal(
        ltt._convert_smart_latex('alpha_beta'),
        r'$\alpha_{\beta}$'
        )
    assert_equal(
        ltt._convert_smart_latex('a_b'),
        r'$a_{b}$'
        )
    assert_equal(
        ltt._convert_smart_latex('a_b_c'),
        r'a $b_{c}$'
        )
    assert_equal(
        ltt._convert_smart_latex('a'),
        r'a'
        )
    assert_equal(
        ltt._convert_smart_latex('a_alpha_beta'),
        r'a $\alpha_{\beta}$'
        )


import warnings
def draw_profile(self, vname, x, y, s=None, band=True, text=True):
    from matplotlib import pyplot as plt
    import numpy as np

    x = np.array(x)
    y = np.array(y)
    #print x
    if s is not None:
        s = np.array(s, dtype=bool)
        x = x[s]
        y = y[s]

    plt.plot(x, y)
    plt.grid(True)
    plt.xlabel(vname)
    plt.ylabel('FCN')


    try:
        minpos = np.argmin(y)
        ymin = y[minpos]
        tmpy = y - ymin
        #now scan for minpos to the right until greater than one
        up = self.errordef
        righty = np.power(tmpy[minpos:] - up, 2)
        #print righty
        right_min = np.argmin(righty)
        rightpos = right_min + minpos
        lefty = np.power((tmpy[:minpos] - up), 2)
        left_min = np.argmin(lefty)
        leftpos = left_min
        #print leftpos, rightpos
        #print x, y
        le = x[minpos] - x[leftpos]
        re = x[rightpos] - x[minpos]
        #print (le, re)

        if band:
            plt.axvspan(x[leftpos], x[rightpos], facecolor='g', alpha=0.5)

        if text:
            plt.figtext(0.5, 0.5,
                '%s = %7.3e ( -%7.3e , +%7.3e)' % (vname, x[minpos], le, re),
                ha='center')
    except ValueError:
        warnings.warn(RuntimeWarning('band and text is requested but'
                                     ' the bound is too narrow.'))

    return x, y, s


def draw_contour(self, x, y, bins=20, bound=2, args=None, show_sigma=False):
#def draw_contour(self, var1, var2, bins=12, bound1=None, bound2=None, lh=True):
    from matplotlib import pyplot as plt
    vx, vy, vz = self.contour(x, y, bins, bound, args, subtract_min=True)
    #x1s, x2s, y = val_contour2d(fit, m, var1, var2, bins=bins,
    #                            bound1=bound1, bound2=bound2)
    #y -= np.min(y)

    v = [self.errordef*(i**2) for i in range(1,4)]

    CS = plt.contour(vx, vy, vz, v, colors=['b', 'k', 'r'])
    if not show_sigma:
        plt.clabel(CS, v)
    else:
        tmp = dict((vv,r'%i $\sigma$'%(i+1)) for i,vv in enumerate(v))
        plt.clabel(CS, v, fmt=tmp, fontsize=16)
    plt.xlabel(x)
    plt.ylabel(y)
    plt.axhline(self.values[y], color='k', ls='--')
    plt.axvline(self.values[x], color='k', ls='--')
    plt.grid(True)
    return vx, vy, vz

def mncontour_grid(self, x, y, numpoints=20, nsigma=2, sigma_res=4, bins=100, edges=False):
    import numpy as np
    from matplotlib import mlab
    dfcn = []
    xps = []
    yps = []
    sigmas = np.linspace(0.1,nsigma+0.5,sigma_res*nsigma)
    for i, this_sig in enumerate(sigmas):
        xminos, yminos, pts = self.mncontour(x, y, numpoints=numpoints,
                                                sigma=this_sig)
        if len(pts)==0:
            warnings.warn(RuntimeWarning('Fail mncontour for %s, %s, sigma=%f'%(x,y,this_sig)))
            continue
        
        xp, yp = zip(*pts)
        xps.append(xp)
        yps.append(yp)
        dfcn.append([this_sig]*len(pts))

    #add the minimum in
    xps.append([self.values[x]])
    yps.append([self.values[y]])
    dfcn.append([0])

    #making grid data x ,y z
    fx, fy, fz = np.hstack(xps), np.hstack(yps), np.hstack(dfcn)

    maxx ,minx = max(fx), min(fx)
    maxy, miny = max(fy), min(fy)

    #extend bound a bit
    maxx += 0.05*(maxx-minx)
    minx -= 0.05*(maxx-minx)
    maxy += 0.05*(maxy-miny)
    miny -= 0.05*(maxy-miny)

    #need constant spacing for linear
    xgrid = np.linspace(minx,maxx, bins)
    ygrid = np.linspace(miny,maxy, bins)
    xstep = (maxx-minx)/(1.0*bins)
    ystep = (maxy-miny)/(1.0*bins)
    #xgrid = np.arange(minx, maxx+xstep/2, xstep) # over cover
    #ygrid = np.arange(miny, maxy+ystep/2, ystep)

    g = mlab.griddata(fx,fy,fz,xgrid,ygrid)

    if edges: #return grid edges instead of mid point (for pcolor)
        xgrid -= xstep/2.
        ygrid -= ystep/2.
        np.resize(xgrid,len(xgrid)+1)
        np.resize(ygrid,len(ygrid)+1)
        xgrid[-1] = xgrid[-2]+xstep/2.
        ygrid[-1] = ygrid[-2]+ystep/2.

    return xgrid, ygrid, g, (xps, yps, dfcn)

def draw_mncontour(self, x, y, bins=100, nsigma=2, numpoints=20, sig_res=4):

    from matplotlib import pyplot as plt

    xgrid, ygrid, g, r = mncontour_grid(self, x, y, numpoints, nsigma, sig_res, bins)
    #g[g.mask] = nsigma+1 #remove the mask

    CS = plt.contour(xgrid, ygrid, g, range(1,nsigma+1))
    plt.clabel(CS, inline=1, fontsize=10)
    plt.xlabel(x)
    plt.ylabel(y)
    return xgrid, ygrid, g, CS



__all__ = [
    'describe',
    'Struct',
    'fitarg_rename',
    'true_param',
    'param_name',
    'extract_iv',
    'extract_limit',
    'extract_error',
    'extract_fix',
    'remove_var',
    'arguments_from_docstring',
]
import inspect
import StringIO
import re


class Struct:
    def __init__(self, **kwds):
        self.__dict__.update(kwds)

    def __str__(self):
        return self.__dict__.__str__()

    def __repr__(self):
        return self.__str__()

    def __getitem__(self, s):
        return self.__dict__[s]


def arguments_from_docstring(doc):
    """Parse first line of docstring for argument name

    Docstring should be of the form ``min(iterable[, key=func])``.

    It can also parse cython docstring of the form
    ``Minuit.migrad(self[, int ncall_me =10000, resume=True, int nsplit=1])``
    """
    if doc is None:
        raise RuntimeError('__doc__ is None')
    sio = StringIO.StringIO(doc.lstrip())
    #care only the firstline
    #docstring can be long
    line = sio.readline()
    if line.startswith("('...',)"):
        line=sio.readline()#stupid cython
    p = re.compile(r'^[\w|\s.]+\(([^)]*)\).*')
    #'min(iterable[, key=func])\n' -> 'iterable[, key=func]'
    sig = p.search(line)
    if sig is None:
        return []
    # iterable[, key=func]' -> ['iterable[' ,' key=func]']
    sig = sig.groups()[0].split(',')
    ret = []
    for s in sig:
        #print s
        #get the last one after all space after =
        #ex: int x= True
        tmp = s.split('=')[0].split()[-1]
        #clean up non _+alphanum character
        ret.append(''.join(filter(lambda x: str.isalnum(x) or x=='_', tmp)))
        #re.compile(r'[\s|\[]*(\w+)(?:\s*=\s*.*)')
        #ret += self.docstring_kwd_re.findall(s)
    ret = filter(lambda x: x!='', ret)

    if len(ret)==0:
        raise RuntimeError('Your doc is unparsable\n'+doc)

    return ret


def is_bound(f):
    """test whether f is bound function"""
    return getattr(f, 'im_self', None) is not None


def dock_if_bound(f, v):
    """dock off self if bound function is passed"""
    return v[1:] if is_bound(f) else v


def better_arg_spec(f, verbose=False):
    """extract function signature

    ..seealso::

        :ref:`function-sig-label`
    """

    try:
        vnames = f.func_code.co_varnames
        #bound method and fake function will be None
        if is_bound(f):
            #bound method dock off self
            return list(vnames[1:f.func_code.co_argcount])
        else:
            #unbound and fakefunc
            return list(vnames[:f.func_code.co_argcount])
    except Exception as e:
        if verbose:
            print e #this might not be such a good dea.
            print "f.func_code.co_varnames[:f.func_code.co_argcount] fails"
        #using __call__ funccode

    try:
        #vnames = f.__call__.func_code.co_varnames
        return list(f.__call__.func_code.co_varnames[1:f.__call__.func_code.co_argcount])
    except Exception as e:
        if verbose:
            print e #this too
            print "f.__call__.func_code.co_varnames[1:f.__call__.func_code.co_argcount] fails"

    try:
        return list(inspect.getargspec(f.__call__)[0][1:])
    except Exception as e:
        if verbose:
            print e
            print "inspect.getargspec(f)[0] fails"

    try:
        return list(inspect.getargspec(f)[0])
    except Exception as e:
        if verbose:
            print e
            print "inspect.getargspec(f)[0] fails"

    #now we are parsing __call__.__doc__
    #we assume that __call__.__doc__ doesn't have self
    #this is what cython gives
    try:
        t = arguments_from_docstring(f.__call__.__doc__)
        if t[0]=='self':
            t = t[1:]
        return t
    except Exception as e:
        if verbose:
            print e
            print "fail parsing __call__.__doc__"

    #how about just __doc__
    try:
        t = arguments_from_docstring(f.__doc__)
        if t[0]=='self':
            t = t[1:]
        return t
    except Exception as e:
        if verbose:
            print e
            print "fail parsing __doc__"

    raise TypeError("Unable to obtain function signature")
    return None

def describe(f, verbose=False):
    """try to extract function arguements name

    .. seealso::

        :ref:`function-sig-label`
    """
    return better_arg_spec(f, verbose)


def fitarg_rename(fitarg, ren):
    """
    rename variable names in fitarg with rename function

    ::

        #simple renaming
        fitarg_rename({'x':1, 'limit_x':1, 'fix_x':1, 'error_x':1},
            lambda pname: 'y' if pname=='x' else pname)
        #{'y':1, 'limit_y':1, 'fix_y':1, 'error_y':1},

        #prefixing
        figarg_rename({'x':1, 'limit_x':1, 'fix_x':1, 'error_x':1},
            lambda pname: 'prefix_'+pname)
        #{'prefix_x':1, 'limit_prefix_x':1, 'fix_prefix_x':1, 'error_prefix_x':1}

    """
    tmp = ren
    if isinstance(ren, basestring):
        ren = lambda x: tmp + '_' + x
    ret = {}
    prefix = ['limit_', 'fix_', 'error_', ]
    for k, v in fitarg.items():
        vn = k
        pf = ''
        for p in prefix:
            if k.startswith(p):
                vn = k[len(p):]
                pf = p
        newvn = pf + ren(vn)
        ret[newvn] = v
    return ret


def true_param(p):
    """check if p is parameter name not a limit/error/fix attributes"""
    return not p.startswith('limit_') and\
        not p.startswith('error_') and\
        not p.startswith('fix_')


def param_name(p):
    """
    extract parameter name from attributes eg

        fix_x -> x
        error_x -> x
        limit_x -> x
    """
    prefix = ['limit_', 'error_', 'fix_']
    for prf in prefix:
        if p.startswith(prf):
            return p[len(prf):]
    return p


def extract_iv(b):
    """extract initial value from fitargs dictionary"""
    return dict((k, v) for k, v in b.items() if true_param(k))


def extract_limit(b):
    """extract limit from fitargs dictionary"""
    return dict((k, v) for k, v in b.items() if k.startswith('limit_'))


def extract_error(b):
    """extract error from fitargs dictionary"""
    return dict((k, v) for k, v in b.items() if k.startswith('error_'))


def extract_fix(b):
    """extract fix attribute from fitargs dictionary"""
    return dict((k, v) for k, v in b.items() if k.startswith('fix_'))


def remove_var(b, exclude):
    """exclude variable in exclude list from b"""
    return dict((k, v) for k, v in b.items() if param_name(k) not in exclude)


def make_func_code(params=None):
    """make a func_code object to fake function signature

        you can make a funccode from describeable object by::

            make_func_code(describe(f))
    """
    return Struct(co_varnames=params, co_argcount=len(params))



import warnings


class IMinuitWarning(RuntimeWarning):
    pass


class InitialParamWarning(IMinuitWarning):
    pass


class HesseFailedWarning(IMinuitWarning):
    pass


warnings.simplefilter('always', InitialParamWarning, append=True)



from libcpp.vector cimport vector
from libcpp.utility cimport pair
from libcpp cimport bool

cdef extern from "<memory>" namespace "std":
    cdef cppclass auto_ptr[T]:
        auto_ptr()
        auto_ptr(T* ptr)
        T* get()

#LCG Minuit
cdef extern from "Minuit/FCNBase.h":
    cdef cppclass FCNBase:
        double call "operator()" (vector[double] x) except+
        double errorDef()

cdef extern from "Minuit/MnApplication.h":
    cdef cppclass MnApplication:
         FunctionMinimum call "operator()" (int,double) except+

cdef extern from "Minuit/MinuitParameter.h":
    cdef cppclass MinuitParameter:
        MinuitParameter(unsigned int, char*, double)
        unsigned int number()
        char* name()
        double value()
        double error()
        bint isConst()
        bint isFixed()

        bint hasLimits()
        bint hasLowerLimit()
        bint hasUpperLimit()
        double lowerLimit()
        double upperLimit()

cdef extern from "Minuit/MnUserCovariance.h":
    cdef cppclass MnUserCovariance:
        unsigned int nrow()
        double get "operator()" (unsigned int row, unsigned int col)

cdef extern from "Minuit/MnGlobalCorrelationCoeff.h":
    cdef cppclass MnGlobalCorrelationCoeff:
        vector[double] globalCC()
        bint isValid()

cdef extern from "Minuit/MnUserParameterState.h":
    cdef cppclass MnUserParameterState:
        MnUserParameterState()
        MnUserParameterState(MnUserParameterState mpst)
        vector[double] params
        void add(char* name, double val, double err)
        void add(char* name, double val, double err, double , double)
        void add(char*, double)

        vector[MinuitParameter] minuitParameters()
        #MnUserParameters parameters()
        MnUserCovariance covariance()
        MnGlobalCorrelationCoeff globalCC()

        double fval()
        double edm()
        unsigned int nfcn()

        void fix(char*)
        void release(char*)
        void setValue(char*, double)
        void setError(char*, double)
        void setLimits(char*, double, double)
        void setUpperLimit(char*, double)
        void setLowerLimit(char*, double)
        void removeLimits(char*)

        bint isValid()
        bint hasCovariance()
        bint hasGlobalCC()

        double value(char*)
        double error(char*)

        unsigned int index(char*)
        char* name(unsigned int)

cdef extern from "Minuit/MnStrategy.h":
    cdef cppclass MnStrategy:
        MnStrategy(unsigned int)


cdef extern from "Minuit/MnMigrad.h":
    cdef cppclass MnMigrad(MnApplication):
        MnMigrad(FCNBase fcn, MnUserParameterState par, MnStrategy str ) except+
        FunctionMinimum call "operator()" (int,double) except+

cdef extern from "Minuit/MnHesse.h":
    cdef cppclass MnHesse:
        MnHesse(unsigned int stra)
        MnUserParameterState call "operator()" (FCNBase , MnUserParameterState, unsigned int maxcalls=0) except+

cdef extern from "Minuit/MnMinos.h":
    cdef cppclass MnMinos:
        MnMinos(FCNBase fcn, FunctionMinimum min, unsigned int stra)
        MinosError minos(unsigned int par, unsigned int maxcalls) except +

cdef extern from "Minuit/MinosError.h":
    cdef cppclass MinosError:
        double lower()
        double upper()
        bint isValid()
        bint lowerValid()
        bint upperValid()
        bint atLowerLimit()
        bint atUpperLimit()
        bint atLowerMaxFcn()
        bint atUpperMaxFcn()
        bint lowerNewMin()
        bint upperNewMin()
        unsigned int nfcn()
        double min()

cdef extern from "Minuit/FunctionMinimum.h":
    cdef cppclass FunctionMinimum:
        FunctionMinimum(FunctionMinimum)
        MnUserParameterState userState()
        MnUserCovariance userCovariance()
        # const_MinimumParameter parameters()
        # const_MinimumError error()

        double fval()
        double edm()
        int nfcn()

        double up()
        bint hasValidParameters()
        bint isValid()
        bint hasValidCovariance()
        bint hasAccurateCovar()
        bint hasPosDefCovar()
        bint hasMadePosDefCovar()
        bint hesseFailed()
        bint hasCovariance()
        bint hasReachedCallLimit()
        bint isAboveMaxEdm()

cdef extern from "Minuit/VariableMetricBuilder.h":
    void set_migrad_print_level "VariableMetricBuilder::setPrintLevel" (int p)

cdef extern from "Minuit/MnContours.h":
    cdef cppclass MnContours:
        MnContours(FCNBase fcn, FunctionMinimum fm, unsigned int stra)
        ContoursError contour(unsigned int, unsigned int, unsigned int npoints)

cdef extern from "Minuit/ContoursError.h":
    cdef cppclass ContoursError:
        ContoursError()
        vector[pair[double, double]] points()
        MinosError xMinosError()
        MinosError yMinosError()



__all__ = [
    'Minuit',
    'describe',
    'Struct',
    'InitialParamWarning',
    '__version__'
]
from _libiminuit import *
from util import describe, Struct
from iminuit_warnings import *
from info import __version__



class ConsoleFrontend:
    """Minuit console front end class
    This class print stuff directly via print
    #TODO: add some color
    """
    def print_fmin(self, sfmin, tolerance=None, ncalls=0):
        """display function minimum information
        for FunctionMinimumStruct *sfmin*.
        It contains various migrad status.
        """
        fmin = sfmin
        goaledm = 0.0001*tolerance*fmin.up if tolerance is not None else ''
        #despite what the doc said the code is actually 1e-4
        #http://wwwasdoc.web.cern.ch/wwwasdoc/hbook_html3/node125.html
        flatlocal = dict(locals().items()+fmin.__dict__.items())
        info1 = 'fval = %(fval)r | total call = %(ncalls)r | ncalls = %(nfcn)r\n' %\
                flatlocal
        info2 = 'edm = %(edm)r (Goal: %(goaledm)r) | up = %(up)r\n' % flatlocal
        header1 = '|' + (' %14s |'*5) % (
                    'Valid',
                    'Valid Param',
                    'Accurate Covar',
                    'Posdef',
                    'Made Posdef')+'\n'
        hline = '-'*len(header1)+'\n'
        status1 = '|' + (' %14r |'*5) % (
                    fmin.is_valid,
                    fmin.has_valid_parameters,
                    fmin.has_accurate_covar,
                    fmin.has_posdef_covar,
                    fmin.has_made_posdef_covar)+'\n'
        header2 = '|' + (' %14s |'*5) % (
                    'Hesse Fail',
                    'Has Cov',
                    'Above EDM',
                    '',
                    'Reach calllim')+'\n'
        status2 = '|' + (' %14r |'*5) % (
                    fmin.hesse_failed,
                    fmin.has_covariance,
                    fmin.is_above_max_edm,
                    '',
                    fmin.has_reached_call_limit)+'\n'

        print hline + info1 + info2 +\
            hline + header1 + hline + status1 +\
            hline + header2 + hline+ status2 +\
            hline

    def print_merror(self, vname, smerr):
        """print minos error for varname"""
        stat = 'VALID' if smerr.is_valid else 'PROBLEM'

        summary = 'Minos Status for %s: %s\n'%\
                (vname, stat)

        error = '| {0:^15s} | {1: >12g} | {2: >12g} |\n'.format(
                    'Error',
                    smerr.
                    lower,
                    smerr.upper)
        valid = '| {0:^15s} | {1:^12s} | {2:^12s} |\n'.format(
                    'Valid',
                    str(smerr.lower_valid),
                    str(smerr.upper_valid))
        at_limit='| {0:^15s} | {1:^12s} | {2:^12s} |\n'.format(
                    'At Limit',
                    str(smerr.at_lower_limit),
                    str(smerr.at_upper_limit))
        max_fcn='| {0:^15s} | {1:^12s} | {2:^12s} |\n'.format(
                    'Max FCN',
                    str(smerr.at_lower_max_fcn),
                    str(smerr.at_upper_max_fcn))
        new_min='| {0:^15s} | {1:^12s} | {2:^12s} |\n'.format(
                    'New Min',
                    str(smerr.lower_new_min),
                    str(smerr.upper_new_min))
        hline = '-'*len(error)+'\n'
        print hline +\
              summary +\
              hline +\
              error +\
              valid +\
              at_limit +\
              max_fcn +\
              new_min +\
              hline

    def print_param(self, mps, merr=None, float_format=None):
        """Print parameter states

        Arguments:

            *mps*: list of MinuitParameter struct

            *merr*: dictionary of vname->minos error struct

            *float_format*: ignored
        """
        def lud(m, k, d):
            #lookup map with default
            return m[k] if k in m else d
        merr = {} if merr is None else merr
        vnames = [mp.name for mp in mps]
        maxlength = max([len(x) for x in vnames])
        maxlength = max(5, maxlength)

        header = (('| {0:^4s} | {1:^%ds} | {2:^8s} | {3:^8s} | {4:^8s} |'
                  ' {5:^8s} | {6:^8s} | {7:^8s} | {8:^8s} |\n')%maxlength).format(
                    '', 'Name', 'Value', 'Para Err',
                    "Err-", "Err+", "Limit-", "Limit+", " ")
        hline = '-'*len(header)+'\n'
        linefmt = ('| {0:>4d} | {1:>%ds} = {2:<8s} | {3:<8s} | {4:<8s} |'
                  ' {5:<8s} | {6:<8s} | {7:<8s} | {8:^8s} |\n')%maxlength
        nfmt = '{0:< 8.4G}'
        nformat = nfmt.format
        blank = ' '*8

        ret = hline+header+hline
        for i, (v, mp) in enumerate(zip(vnames, mps)):
            tmp = [i, v]

            tmp.append(nfmt.format(mp.value))
            tmp.append(nfmt.format(mp.error))

            tmp.append(nformat(merr[v].lower) if v in merr else blank)
            tmp.append(nformat(merr[v].upper) if v in merr else blank)

            tmp.append(nformat(mp.lower_limit) if mp.has_limits else blank)
            tmp.append(nformat(mp.upper_limit) if mp.has_limits else blank)

            tmp.append(
                'FIXED' if mp.is_fixed else 'CONST' if mp.is_const else '')

            line = linefmt.format(*tmp)
            ret+=line
        ret+=hline
        print ret

    def print_banner(self, cmd):
        """show banner of command"""
        ret = '*'*50+'\n'
        ret += '*{:^48}*'.format(cmd)+'\n'
        ret += '*'*50+'\n'
        print ret

    def print_matrix(self, vnames, matrix):
        """TODO: properly implement this"""
        print vnames
        print matrix
        maxlen = max(len(v) for v in vnames)
        narg = len(matrix)
        vfmt = '%%%ds'%maxlen
        vblank = ' '*maxlen
        fmt = '%3.2f ' # 4char
        dfmt = '%4d '
        tmp = ''
        header = vblank+' '*4+'  | '+(dfmt*narg)%tuple(range(narg))+'\n'
        blank_line = '-'*len(header)+'\n'
        tmp += header + blank_line
        for i, (v, row) in enumerate(zip(vnames, matrix)):
            fmt = '%3.2f '*narg
            head = (vfmt+' %4d | ')%(v, i)
            content = (fmt)%tuple(row)+'\n'
            tmp += head + content
        tmp += blank_line
        print tmp

    def print_hline(self):
        print '*'*70



# -*- coding: utf-8 -*-
# <nbformat>3.0</nbformat>

# <codecell>

from iminuit import Minuit

# <codecell>

def f(x,y,z):
    return x**2+(y-x)**2+z**2

# <codecell>

m = Minuit(f)
m.migrad()

# <codecell>

m.print_matrix()

# <codecell>

x,y,s = m.mnprofile('y')

# <codecell>

plot(x,y)

# <codecell>

def f(x, y, z):
    return (x-1)**2 + (y-x)**2 + (z-2*x)**2

def make_mncontour():
    m = Minuit(f)
    m.migrad()
    m.draw_mncontour('x', 'y')
    plt.savefig('images/mncontour.png')

# <codecell>

make_mncontour()

# <codecell>

,

# <codecell>





# -*- coding: utf8 -*-
from distutils.core import setup, Extension
from os.path import dirname, join
from glob import glob

#static link
cwd = dirname(__file__)
minuit_src = glob(join(cwd, 'Minuit/src/*.cpp'))
minuit_header = join(cwd, 'Minuit')

libiminuit = Extension('iminuit._libiminuit',
                    sources=['iminuit/_libiminuit.cpp'] + minuit_src,
                    include_dirs=[minuit_header],
                    libraries=[],
                    #extra_compile_args=['-Wall', '-Wno-sign-compare',
                    #                      '-Wno-write-strings'],
                    extra_link_args=[])

execfile('iminuit/info.py')

setup(
    name='iminuit',
    version=__version__,
    description='Interactive Minimization Tools based on MINUIT',
    long_description=''.join(open('README.rst').readlines()[4:]),
    author='Piti Ongmongkolkul',
    author_email='piti118@gmail.com',
    url='http://iminuit.github.io/iminuit/',
    download_url='http://pypi.python.org/packages/source/i/'
            'iminuit/iminuit-%s.tar.gz' % __version__,
    package_dir={'iminuit': 'iminuit'},
    packages=['iminuit'],
    ext_modules=[libiminuit],
    classifiers=[
        "Programming Language :: Python",
        'Topic :: Scientific/Engineering :: Physics',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Intended Audience :: Science/Research',
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'License :: OSI Approved :: MIT License'
    ]
)



from nose.tools import *
from iminuit.util import *

def test_fitarg_rename():
    fitarg = {'x':1,'limit_x':(2,3),'fix_x':True,'error_x':10}
    ren = lambda x: 'z_'+x
    newfa = fitarg_rename(fitarg,ren)
    assert_true('z_x' in newfa)
    assert_true('limit_z_x' in newfa)
    assert_true('error_z_x' in newfa)
    assert_true('fix_z_x' in newfa)
    assert_equal(len(newfa),4)

def test_fitarg_rename_strprefix():
    fitarg = {'x':1,'limit_x':(2,3),'fix_x':True,'error_x':10}
    newfa = fitarg_rename(fitarg,'z')
    assert_true('z_x' in newfa)
    assert_true('limit_z_x' in newfa)
    assert_true('error_z_x' in newfa)
    assert_true('fix_z_x' in newfa)
    assert_equal(len(newfa),4)

def test_true_param():
    assert_true(true_param('N'))
    assert_false(true_param('limit_N'))
    assert_false(true_param('error_N'))
    assert_false(true_param('fix_N'))

def test_param_name():
    assert_equal(param_name('N'),'N')
    assert_equal(param_name('limit_N'),'N')
    assert_equal(param_name('error_N'),'N')
    assert_equal(param_name('fix_N'),'N')

def test_extract_iv():
    d = dict(k=1.,limit_k=1.,error_k=1.,fix_k=1.)
    ret = extract_iv(d)
    assert_true('k' in ret)
    assert_false('limit_k' in ret)
    assert_false('error_k' in ret)
    assert_false('fix_k' in ret)

def test_extract_limit():
    d = dict(k=1.,limit_k=1.,error_k=1.,fix_k=1.)
    ret = extract_limit(d)
    assert_false('k' in ret)
    assert_true('limit_k' in ret)
    assert_false('error_k' in ret)
    assert_false('fix_k' in ret)

def test_extract_error():
    d = dict(k=1.,limit_k=1.,error_k=1.,fix_k=1.)
    ret = extract_error(d)
    assert_false('k' in ret)
    assert_false('limit_k' in ret)
    assert_true('error_k' in ret)
    assert_false('fix_k' in ret)

def test_extract_fix():
    d = dict(k=1.,limit_k=1.,error_k=1.,fix_k=1.)
    ret = extract_fix(d)
    assert_false('k' in ret)
    assert_false('limit_k' in ret)
    assert_false('error_k' in ret)
    assert_true('fix_k' in ret)

def test_remove_var():
    dk = dict(k=1,limit_k=1,error_k=1,fix_k=1)
    dl = dict(l=1,limit_l=1,error_l=1,fix_l=1)
    dm = dict(m=1,limit_m=1,error_m=1,fix_m=1)
    dn = dict(n=1,limit_n=1,error_n=1,fix_n=1)
    d = {}
    d.update(dk)
    d.update(dl)
    d.update(dm)
    d.update(dn)

    ret = remove_var(d,['k','m'])
    for k in dk: assert_false(k in ret)
    for k in dl: assert_true(k in ret)
    for k in dm: assert_false(k in ret)
    for k in dn: assert_true(k in ret)

def test_arguments_from_docstring():
    s = 'f(x, y, z)'
    a = arguments_from_docstring(s)
    assert_equal(a, ['x','y','z'])
    #this is a hard one
    s = 'Minuit.migrad( int ncall_me =10000, [resume=True, int nsplit=1])'
    a = arguments_from_docstring(s)
    assert_equal(a, ['ncall_me','resume','nsplit'])



cimport cython
import numpy as np
cimport numpy as np #overwritten those from python with cython
from libc.math cimport exp, M_PI, sqrt, log

@cython.embedsignature(True)
cpdef external_mypdf(double x, double mu, double sigma):
    #cpdef means generate both c function and python function
    cdef double norm = 1./(sqrt(2*M_PI)*sigma)
    cdef double ret = exp(-1*(x-mu)*(x-mu)/(2.*sigma*sigma))*norm
    return ret

cdef class External_LogLH:#cdef is here to reduce name lookup for __call__
    cdef np.ndarray data
    cdef int ndata
    cdef public func_code
    cdef object pdf

    def __init__(self, data):
        self.data = data
        self.ndata = len(data)

    #@cython.boundscheck(False)#you can turn off bound checking
    @cython.embedsignature(True)#you need this for describe to work
    def compute(self, double mu, double sigma):
        cdef np.ndarray[np.double_t, ndim=1] mydata = self.data
        cdef double loglh = 0.
        cdef double thisdata
        for i in range(self.ndata):
            thisdata = mydata[i]
            loglh -= log(external_mypdf(mydata[i],mu,sigma))
        return loglh 



class Gradient:
    #from http://code.activestate.com/recipes/266466-html-colors-tofrom-rgb-tuples/
    @classmethod
    def color_for(cls, v, min=0., max=1., startcolor=(163, 254, 186),
                  stopcolor=(255, 117, 117)):
        c = [0] * 3
        for i, sc in enumerate(startcolor):
            offset = 1.0 * (v-min) * (stopcolor[i]-startcolor[i]) / (max-min)
            c[i] = round(startcolor[i] + offset)
        return tuple(c)

    @classmethod
    def xcolor_for(cls, v):
        c = cls.color_for(abs(v))
        return '[rgb]{%3.2f,%3.2f,%3.2f}' % (c[0]/255., c[1]/255., c[2]/255.)

    @classmethod
    def rgb_color_for(cls, v):
        c = cls.color_for(abs(v))
        return 'rgb(%d,%d,%d)' % c



# -*- coding: utf-8 -*-
# <nbformat>3.0</nbformat>

# <markdowncell>

# Hard Core Tutorial
# ==================
# 
# Typically in fitting, performance matters. Python is slow since it does tons of extra stuff(name lookup etc.) for a good reason. We can fix that with cython and numpy. This tutorial will demonstate how one would write a model which can take data and fit to the data. We will be demonstrating two ways: fastway and generic way. As a bonus we will show how to parallelize your cost function.

# <markdowncell>

# ##Basic Cython
# Before we go on let's talk about how to use cython efficiently. Cython speed things up by using static type where it can. Generally the more type information you tell them the better it can generate C code.
# 
# Cython has a very handy option call annotate which lets you know which line of code is static type which one make a call to python object.

# <codecell>

%pylab inline
%load_ext cythonmagic
from iminuit import Minuit

# <codecell>

%%cython --annotate

def slow_f(n):
    x = 100.
    for i in range(n):
        x+=n
    return x

#you tell it more type information like this
def fast_f(int n):
    cdef double x=100.
    cdef int i
    for i in range(n):
        x+=n
    return x
        

# <markdowncell>

# You can see that there is yellow code line and white code line.
# 
# - yellow code line means calling to python code
# - white code line means native C
# 
# Basically, your goal is to get as many white lines as possible. By telling as much type information as possible.
# 
# You can also click on each line to see what code cython actually generate for you. (You many need to double click it)

# <codecell>

%timeit -n10 -r10 slow_f(100)

# <codecell>

%timeit -n10 -r10 fast_f(100)

# <markdowncell>

# ###Quick And Dirty way
# Let's look at how to write a cython cost function

# <codecell>

%pylab inline
%load_ext cythonmagic
from iminuit import Minuit

# <codecell>

np.random.seed(0)
data = randn(2e6)*3+2 #mu=2, sigma=3
hist(data,bins=100, histtype='step');

# <codecell>

%%cython --force
#use --annotate if you wonder what kind of code it generates
cimport cython
import numpy as np
cimport numpy as np #overwritten those from python with cython
from libc.math cimport exp, M_PI, sqrt, log
from iminuit.util import describe, make_func_code

@cython.embedsignature(True)#dump the signatre so describe works
cpdef double mypdf(double x, double mu, double sigma):
    #cpdef means generate both c function and python function
    cdef double norm = 1./(sqrt(2*M_PI)*sigma)
    cdef double ret = exp(-1*(x-mu)*(x-mu)/(2.*sigma*sigma))*norm
    return ret

cdef class QuickAndDirtyLogLH:#cdef is here to reduce name lookup for __call__
    cdef np.ndarray data
    cdef int ndata
    
    def __init__(self, data):
        self.data = data
        self.ndata = len(data)
    
    @cython.embedsignature(True)#you need this to dump function signature in docstring
    def compute(self, double mu, double sigma):
        #this line is a cast not a copy. Let cython knows mydata will spit out double
        cdef np.ndarray[np.double_t, ndim=1] mydata = self.data
        cdef double loglh = 0.
        cdef double thisdata
        for i in range(self.ndata):
            thisdata = mydata[i]
            loglh -= log(mypdf(mydata[i],mu,sigma))
        return loglh

# <codecell>

describe(mypdf)

# <codecell>

lh = QuickAndDirtyLogLH(data)
describe(lh.compute)

# <codecell>

m = Minuit(lh.compute, mu=1.5, sigma=2.5, error_mu=0.1, 
    error_sigma=0.1, limit_sigma=(0.1,10.0))
m.migrad();

# <codecell>

%%timeit -n2 -r2 #-n1 -r1 
m = Minuit(lh.compute, mu=1.5, sigma=2.5, error_mu=0.1, 
    error_sigma=0.1, limit_sigma=(0.1,10.0),print_level=0)
m.migrad()

# <markdowncell>

# Have your cython PDF in a separate file
# ---------------------------------------
# 
# Lots of time your stuff is incredibly complicated and doesn't fit in ipython notebook. Or you may want to reuse your PDF in many notebooks. We have external_pdf.pyx in the same directory as this tutorial. This is how you load it.

# <codecell>

%pylab inline
%load_ext cythonmagic
from iminuit import Minuit

# <codecell>

import pyximport;
pyximport.install(
    setup_args=dict(
        include_dirs=[np.get_include()],#include directory
    #    libraries = ['m']#'stuff you need to link (no -l)
    #    library_dirs ['some/dir']#library dir
    #    extra_compile_args = ['-g','-O2'],
    #    extra_link_args=['-some-link-flags'],
    ),
    reload_support=True,#you may also find this useful
) #if anything funny is going on look at your console
import external_pdf

# <codecell>

#reload(external_pdf) #you may find this useful for reloading your module

# <codecell>

np.random.seed(0)
data = randn(2e6)*3+2
lh = external_pdf.External_LogLH(data)

# <codecell>

m = Minuit(lh.compute,mu=1.5, sigma=2.5, error_mu=0.1, 
    error_sigma=0.1, limit_sigma=(0.1,10.0))
m.migrad();

# <codecell>

%%timeit -r2 -n2 
m = Minuit(lh.compute,mu=1.5, sigma=2.5, error_mu=0.1, 
    error_sigma=0.1, limit_sigma=(0.1,10.0),print_level=0)
m.migrad()

# <markdowncell>

# ###Generic Reusable Cost Function
# 
# Sometime we want to write a cost function that will take in any pdf and data and compute appropriate
# cost function. This is slower than the previous example but will make your code much more reusable.

# <codecell>

%pylab inline
%load_ext cythonmagic
from iminuit import Minuit

# <codecell>

np.random.seed(0)
data = randn(2e6)*3+2 #mu=2, sigma=3

# <markdowncell>

# This is how you do it

# <codecell>

%%cython
#use --annotate if you wonder what kind of code it generates
cimport cython
import numpy as np
cimport numpy as np #overwritten those from python with cython
from iminuit.util import make_func_code, describe
from libc.math cimport log

cdef class LogLH:#cdef is here to reduce name lookup for __call__(think of struct)
    cdef np.ndarray data
    cdef int ndata
    cdef public func_code
    cdef object pdf
    
    def __init__(self, pdf, data):
        self.data = data
        self.ndata = len(data)
        #the important line is here
        self.func_code = make_func_code(describe(pdf)[1:])#1: dock off independent param
        self.pdf = pdf
    
    #@cython.boundscheck(False)#you can turn off bound checking
    def __call__(self, *args):
        cdef np.ndarray[np.double_t, ndim=1] mydata = self.data#this line is very important
        #with out this line cython will have no idea about data type
        cdef double loglh = 0.
        cdef list larg = [0.]+list(args)
        
        for i in range(self.ndata):
            #it's slower because we need to do so many python stuff
            #to do generic function call
            #if you are python hacker and know how to get around this
            #please let us know
            larg[0] = mydata[i]
            loglh -= log(self.pdf(*larg))
        return loglh

# <markdowncell>

# And your favorite PDF

# <codecell>

%%cython
#use --annotate if you wonder what kind of code it generates
from libc.math cimport exp, M_PI, sqrt, log
cimport cython

@cython.binding(True)
def mypdf(double x, double mu, double sigma): 
    #cpdef means generate both c function and python function
    cdef double norm = 1./(sqrt(2*M_PI)*sigma)
    cdef double ret = exp(-1*(x-mu)*(x-mu)/(2.*sigma*sigma))*norm
    return ret

# <codecell>

mylh = LogLH(mypdf,data)

# <codecell>

print describe(mypdf)
print describe(mylh)

# <codecell>

describe(mypdf)

# <codecell>

m=Minuit(mylh, mu=1.5, sigma=2.5, error_mu=0.1, 
    error_sigma=0.1, limit_sigma=(0.1,10.0))
#you can feel it's much slower than before
m.migrad();

# <codecell>

%%timeit -r2 -n2
m=Minuit(mylh, mu=1.5, sigma=2.5, error_mu=0.1, 
    error_sigma=0.1, limit_sigma=(0.1,10.0), print_level=0)
m.migrad() #you can feel it's much slower than before

# <codecell>

#before
x = linspace(-10,12,100)
before = np.fromiter((mypdf(xx,1.5,2.5) for xx in x), float);
after = np.fromiter((mypdf(xx,m.values['mu'],m.values['sigma']) for xx in x), float);
plot(x,before, label='before')
plot(x,after, label='after')
hist(data, normed=True, bins=100,histtype='step',label='data')
legend();

# <markdowncell>

# ###Parallel processing with `multiprocessing`
# This example is most likely not working on windows. You need to implement a bunch of workaround for lack of fork() on Windows such that it's easier to install a decent linux on your computer.
# 
# The idea here on how to parallelize your cost function is to separate you data into multiple chunks and have each worker calculate your cost function, collect them at the end and add them up.
# 
# The tool we will be showing here is Python multiprocess. We will be rewriting our generic cost funciton but now with multiprocessing.
# 
# You might be worried about that forking process will copy your data. Most modern OS use [Copy On Write](http://en.wikipedia.org/wiki/Copy-on-write mechanism)(look at wiki)
# what this means is that when it forks a process
# it doesn't copy memory there unless you are writing it

# <codecell>

%load_ext cythonmagic
from iminuit import Minuit

# <codecell>

%%cython -f
cimport cython
import numpy as np
cimport numpy as np #overwritten those from python with cython
from libc.math cimport exp, M_PI, sqrt, log, floor
from libc.stdio cimport printf
from iminuit.util import make_func_code, describe
import multiprocessing as mp
from multiprocessing import Manager
import os
#import logging
#logger = multiprocessing.log_to_stderr()
#don't do this in ipython either it will crash(watch ipython #2438)
#logger.setLevel(multiprocessing.SUBDEBUG)

@cython.embedsignature(True)#dump the signatre so describe works
cpdef double mypdf(double x, double mu, double sigma):
    #cpdef means generate both c function and python function
    cdef double norm
    cdef double ret
    norm = 1./(sqrt(2*M_PI)*sigma)
    ret = exp(-1*(x-mu)*(x-mu)/(2.*sigma*sigma))*norm
    return ret

cdef class Multiprocess_LogLH:#cdef is here to reduce name lookup for __call__
    cdef np.ndarray data
    #cdef double* databuffer#i'm not really sure if numpy will do copy on write only or not
    cdef int ndata
    cdef public func_code
    cdef object pdf
    cdef int njobs
    cdef list starts
    cdef list stops
    cdef object pool
    cdef int i
    cdef object manager
    cdef object results
    def __init__(self, pdf, np.ndarray[np.double_t] data, njobs=None):
        self.data = data
        #self.databuffer = <double*>data.data
        self.ndata = len(data)
        
        #the important line is here
        self.func_code = make_func_code(describe(pdf)[1:])#1: dock off independent param
        self.pdf = pdf
        
        #worker pool stuff
        self.njobs = njobs if njobs is not None else mp.cpu_count()
        print 'Number of CPU: ',self.njobs
        #determine chunk size
        chunk_size = floor(self.ndata/self.njobs)
        self.starts = [i*chunk_size for i in range(self.njobs)]
        self.stops = [(i+1)*chunk_size for i in range(self.njobs)]
        self.stops[-1] = self.ndata #add back last couple data from round off
        self.i=0
        self.manager = Manager()
        self.results = self.manager.Queue()
    
    @cython.embedsignature(True)
    cpdef process_chunk(self, 
            int pid, int start, int stop, tuple args, object results): #start stop is [start, stop) 
        #be careful here there is a bug in ipython preventing
        #child process from printing to stdout/stderr (you will get a segfault)
        #I submitted a patch https://github.com/ipython/ipython/pull/2712
        #Ex: For now, do something like this if you need to debug
        #msg = str(('Start Worker:', pid, start, stop,os.getpid()))+'\n'
        #printf(msg)
        #it will run fine as a python script though
        cdef np.ndarray[np.double_t, ndim=1] mydata = self.data#get cython to know the type
        cdef int i
        cdef double loglh = 0.
        cdef double tmp = 0.
        cdef tuple t
        #calculate lh for this chunk
        cdef list larg = [0.]+list(args)
        for i in range(start,stop):
            tmp = mydata[i]
            larg[0] = tmp
            loglh -= log(self.pdf(*larg))
        results.put(loglh) #put result in multiprocessing.queue
        return loglh#return isn't necessary since it will be ignored but useful for testing

    def __call__(self, *args):
        cdef double ret=0
        pool = [mp.Process(target=self.process_chunk,
                           args=(i,self.starts[i],self.stops[i],args,self.results)) 
                    for i in range(self.njobs)]
        #you may think that forking this many time is inefficient
        #We can do better but this is not bad. Since most of the time
        #will be spend on calculating your loglh this is cheap compared to those.
        #If forking is more expensive than what each worker does then... 
        #your problem is something else.
        #Copy on write ensures that data points will never be copied (unless your write to it)
        self.i+=1
        for p in pool: p.start() #start everyone
        for p in pool: p.join() #wait for everyone to finish
        while not self.results.empty(): #collect the result
            tmp = self.results.get()
            ret += tmp
        return ret

# <codecell>

np.random.seed(0)
data = randn(2e6)*3+2 #10 millions
mlh = Multiprocess_LogLH(mypdf,data)

# <codecell>

#good idea to debug it in non-multiprocess environment first
import multiprocessing as mp
q = mp.Queue()
mlh.process_chunk(0,0,10,(0,1),q)

# <codecell>

#then see if it works on one point
mlh(0,1)

# <codecell>

m = Minuit(mlh,mu=1.5, sigma=2.5, error_mu=0.1, 
    error_sigma=0.1, limit_sigma=(0.1,10.0))
m.migrad();

# <codecell>

%%timeit -r2 -n2
m = Minuit(mlh,mu=1.5, sigma=2.5, error_mu=0.1, 
    error_sigma=0.1, limit_sigma=(0.1,10.0),print_level=0)
m.migrad()

# <markdowncell>

# ###Parallel Computing With Cython and OpenMP
# *For this tutorial you will need a compiler with openmp support. GCC has one. However, clang does NOT support it.*
# 
# **This is not recommended. Do this only when overhead of multiprocessing is too big for you. The performance gain is probaly not worth your headache.**
# 
# Computer nowadays are multi-core machines so it makes sense to utilize all of them. This method is fast but quite restricted and cubersome since you need to write function such that cython can figure out its reentrant-ness. And you need some understanding of thread-local and thread-share variable.
# 
# You can read [prange](http://wiki.cython.org/enhancements/prange) from cython wiki for more information and how to gain a more complete control over paralelization. The official documentation is [here](http://docs.cython.org/src/userguide/parallelism.html)

# <codecell>

%load_ext cythonmagic
from iminuit import Minuit

# <codecell>

%%cython -f -c-fopenmp --link-args=-fopenmp -c-g
#use --annotate if you wonder what kind of code it generates
cimport cython
import numpy as np
cimport numpy as np #overwritten those from python with cython
from libc.math cimport exp, M_PI, sqrt, log
from iminuit.util import describe, make_func_code
import multiprocessing
from cython.parallel import prange


#notice nogil a the end (no global intepreter lock)
#cython doesn't do a super thorough check for this
#so make sure your function is reentrant this means approximately 
#just simple function compute simple stuff based on local stuff and no read/write to global 
@cython.embedsignature(True)#dump the signatre so describe works
@cython.cdivision(True)
cpdef double mypdf(double x, double mu, double sigma) nogil:
    #cpdef means generate both c function and python function
    cdef double norm
    cdef double ret
    norm = 1./(sqrt(2*M_PI)*sigma)
    ret = exp(-1*(x-mu)*(x-mu)/(2.*sigma*sigma))*norm
    return ret

cdef class ParallelLogLH:#cdef is here to reduce name lookup for __call__
    cdef np.ndarray data
    cdef int ndata
    cdef int njobs
    cdef np.ndarray buf#buffer for storing result from each job
    def __init__(self, data, njobs=None):
        self.data = data
        self.ndata = len(data)
        self.njobs = njobs if njobs is not None else multiprocessing.cpu_count()
        print 'Number of CPU: ',self.njobs
    
    @cython.boundscheck(False)
    @cython.embedsignature(True)#you need this to dump function signature in docstring
    def compute(self, double mu, double sigma):
        cdef np.ndarray[np.double_t, ndim=1] mydata = self.data
        cdef double loglh = 0.
        cdef tuple t
        cdef double thisdata
        cdef int i=0
        #in parallel computing you need to be careful which variable is
        #thread private which variable is shared between thread
        #otherwise you will get into hard to detect racing condition
        #cython rule of thumb(guess rule) is
        # 1) assigned before use is thread private
        # 2) read-only is thread-shared
        # 3) inplace modification only is thread shared
        cdef int njobs = self.njobs
        cdef double tmp
        with nogil:
            for i in prange(self.ndata, 
                            num_threads=njobs, 
                            chunksize=100000, 
                            schedule='dynamic'):#split into many threads
                thisdata = mydata[i] #this is assigned before read so it's thread private
                tmp  = mypdf(thisdata,mu,sigma) #also here assigne before read
                loglh -= log(tmp) #inplace modification so loglh is thread shared
        return loglh

# <codecell>

np.random.seed(0)
data = randn(2e6)*3+2

# <codecell>

plh = ParallelLogLH(data)

# <codecell>

plh.compute(1.5,2.0)

# <codecell>

describe(plh.compute)

# <codecell>

m=Minuit(plh.compute,mu=1.5, sigma=2.5, error_mu=0.1, 
    error_sigma=0.1, limit_sigma=(0.1,10.0),print_level=0)
m.migrad();

# <codecell>

%%timeit -n2 -r2
m=Minuit(plh.compute,mu=1.5, sigma=2.5, error_mu=0.1, 
    error_sigma=0.1, limit_sigma=(0.1,10.0),print_level=0)
m.migrad()

# <markdowncell>

# ###OpenCL
# **Want to do fitting using parallel GPU core? If you have multicore GPU Cluster/Machine figure out how to do it and let me know.**
# 
# I don't have the machine with such capability so I never bother....Interested in buying one for me? :P

